import json
import requests
from requests.auth import HTTPBasicAuth

user = "kbtoku"
pw = "abcxyz"

service_type = "ingest"

endpoint = "/services/available.json?serviceType=org.opencastproject." + str(
    service_type
)
url = "%s%s" % ("http://opencast-dev.bibliothek.kit.edu:8080", endpoint)

print(url)
res = requests.get(url, auth=HTTPBasicAuth(user, pw))

if res.ok:
    print(res.json())

    services = (json.loads(res.content).get("services") or {}).get("service", [])

else:
    print("FAIL!")



import logging
import unittest
import pytest

from ocpy import OcPyRequestException
from ocpy.api.admin_ui.themes import ThemesApi


@pytest.mark.usefixtures("setup_oc_connection")
class TestThemesClass(unittest.TestCase):
    def test_get_services(self):
        themes_api = ThemesApi()
        self.assertIsNotNone(themes_api.get_all_themes())

    def test_delete_theme(self):
        themes_api = ThemesApi()
        with self.assertRaises(OcPyRequestException) as context:
            themes_api.delete_theme(234345873459)  # non existing theme
        self.assertIn("could not delete theme", str(context.exception))



import yaml
from random import choice
from loguru import logger

from ocpy.api.service import ServiceApi, ServiceType
from ocpy.api.ingest import IngestApi
from ocpy.api.assets import AssetsApi
from ocpy.api.external.base import BaseApi
from ocpy.api.external.agents import AgentsApi
from ocpy.api.external.events import EventsApi
from ocpy.api.external.series import SeriesApi
from ocpy.api.external.workflows import WorkflowsApi
from ocpy.api.recordings import RecordingsApi

# from .series import GroupsApi
# from .series import SecurityApi

from ocpy import get_connection_config


def get_base_api_client() -> BaseApi:
    connection_config = get_connection_config()
    if connection_config is None:
        raise Exception(
            "please call >setup_connection_config< before using the API client(s)"
        )
    else:
        return BaseApi(**connection_config)


def get_agents_api_client() -> AgentsApi:
    connection_config = get_connection_config()
    if connection_config is None:
        raise Exception(
            "please call >setup_connection_config< before using the API client(s)"
        )
    else:
        return AgentsApi(**connection_config)


def get_events_api_client() -> EventsApi:
    connection_config = get_connection_config()
    if connection_config is None:
        raise Exception(
            "please call >setup_connection_config< before using the API client(s)"
        )
    else:
        return EventsApi(**connection_config)


def get_series_api_client() -> SeriesApi:
    connection_config = get_connection_config()
    if connection_config is None:
        raise Exception(
            "please call >setup_connection_config< before using the API client(s)"
        )
    else:
        return SeriesApi(**connection_config)


def get_workflows_api_client() -> WorkflowsApi:
    connection_config = get_connection_config()
    if connection_config is None:
        raise Exception(
            "please call >setup_connection_config< before using the API client(s)"
        )
    else:
        return WorkflowsApi(**connection_config)


def get_recordings_api_client() -> RecordingsApi:
    connection_config = get_connection_config()
    if connection_config is None:
        raise Exception(
            "please call >setup_connection_config< before using the API client(s)"
        )
    else:
        return RecordingsApi(**connection_config)


def get_service_api_client() -> ServiceApi:
    connection_config = get_connection_config()
    if connection_config is None:
        raise Exception(
            "please call >setup_connection_config< before using the API client(s)"
        )
    else:
        return ServiceApi(**connection_config)


def get_ingest_api_client() -> IngestApi:
    connection_config = get_connection_config()
    if connection_config is None:
        raise Exception(
            "please call >setup_connection_config< before using the API client(s)"
        )
    else:
        service_api = get_service_api_client()
        ingest_service = choice(service_api.get_available(ServiceType.ingest))
        return IngestApi(ingest_service.get_url(), **connection_config)

def get_assets_api_client() -> AssetsApi:
    connection_config = get_connection_config()
    if connection_config is None:
        raise Exception(
            "please call >setup_connection_config< before using the API client(s)"
        )
    else:
        return AssetsApi(**connection_config)




import requests
from requests.auth import HTTPBasicAuth
import json
from pprint import pprint

from typing import List

from ocpy.api.api_client import OpenCastBaseApiClient


class User:
    def __init__(self, user, password, url, data):
        self.base_url = url
        self.user = user
        self.password = password
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_roles(self, **kwargs) -> list:
        url = self.base_url + "/roles"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.json()
        raise Exception("Could not get user roles!")

    def get_email(self) -> str:
        return self.data["email"]

    def get_name(self) -> str:
        return self.data["name"]

    def get_provider(self) -> str:
        return self.data["provider"]

    def get_username(self) -> str:
        return self.data["username"]

    def get_user_role(self) -> str:
        return self.data["userrole"]


class Organization:
    def __init__(self, user, password, url, data):
        self.base_url = url
        self.user = user
        self.password = password
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_admin_role(self) -> str:
        return self.data["adminRole"]

    def get_anonymous_role(self) -> str:
        return self.data["anonymousRole"]

    def get_id(self) -> List[str]:
        return self.data["id"]

    def get_name(self) -> str:
        return self.data["name"]

    def get_properties(self, **kwargs) -> list:
        url = self.base_url + "/properties"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.json()
        raise Exception("Could not get user roles!")


class BaseApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/api"

    def get_api_info(self, **kwargs) -> dict:
        res = requests.get(
            self.base_url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.json()
        raise Exception("Could not get api info!")

    def get_api_version(self, **kwargs) -> dict:
        url = self.base_url + "/version"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.json()
        raise Exception("Could not get api version!")

    def get_default_api_version(self, **kwargs) -> dict:
        url = self.base_url + "/version/default"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.json()
        raise Exception("Could not get default api version!")

    def get_user_info(self, **kwargs) -> User:
        url = self.base_url + "/info/me"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            user = User(self.user, self.password, url, res.json())
            return user
        raise Exception("Could not get user info!")

    def get_organization_info(self, **kwargs) -> Organization:
        url = self.base_url + "/info/organization"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            orga = Organization(self.user, self.password, url, res.json())
            return orga
        raise Exception("Could not get organization info!")

    def clear_index(self, **kwargs):
        """Clears the index for the external API. This is useful if the API returns events that don't exist anymore.
        There is another clearIndex function for the admin ui. Also the engage player uses another endpoint
        (the search endpoint) which can't be cleared, but single items can be deleted from search using the search
        endpoint's delete function."""
        url = self.base_url + "/clearIndex"
        res = requests.post(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return "ok"
        raise Exception("Could not get clear index!")

    def recreate_index(self, service=None, **kwargs):
        """Recreates the index for the external API (optionally for a specific service). Allowed services are:
        Groups, Acl, Themes, Series, Scheduler, Workflow, AssetManager and Comments.
        The service order (see above) is very important! Make sure, you do not run index rebuild for
        more than one service at a time!
        The service parameter can be omitted.

        This is useful if the API returns events that don't exist anymore.
        There is another recreateIndex function for the admin ui. Also the engage player uses another endpoint
        (the search endpoint) which can't be recreated, but single items can be deleted from search using the search
        endpoint's delete function."""
        url = self.base_url + "/recreateIndex"
        if service is not None:
            url += "/" + service
        res = requests.post(url, auth=HTTPBasicAuth(self.user, self.password), **kwargs)
        if res.ok:
            return "ok"
        raise Exception("Could not get clear index!")


def main():
    api = BaseApi()
    info = api.get_api_info()
    pprint(info)
    pprint(api.get_api_version())
    pprint(api.get_default_api_version())
    user = api.get_user_info()
    pprint(user)
    pprint(user.get_roles())
    pprint(api.get_organization_info())
    pprint(api.get_organization_info().get_properties())
    print(api.clear_index())
    print(api.recreate_index())


if __name__ == "__main__":
    main()



#  Copyright (c) 2019. Tobias Kurze

import json
from typing import List, Optional, Union

from ocpy.utils import camel_case_to_snake_case


class EventMetadataItem:
    def __init__(
        self,
        identifier: str,
        label: str,
        required: bool,
        read_only: bool,
        type: str,
        value: Union[str, int],
    ):
        self.id = identifier
        self.label = label
        self.read_only = read_only
        self.required = required
        self.type = type
        self.value = value

    def __str__(self):
        obj = self.get_as_dict()
        return json.dumps(obj, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_as_dict(self):
        return {
            "id": self.id,
            "label": self.label,
            "readOnly": self.read_only,
            "required": self.required,
            "type": self.type,
            "value": self.value,
        }


class EventMetadata:
    def __init__(
        self,
        metadata_item: Union[EventMetadataItem, List[EventMetadataItem]],
        flavor: str = "dublincore/episode",
        title: str = "EVENTS.EVENTS.DETAILS.CATALOG.EPISODE",
    ):
        self.flavor = flavor
        self.title = title
        self.meta_data_items = []
        self.add_metadata_item(metadata_item)

    def __str__(self):
        obj = self.get_metadata_as_dict(compact=False)
        return json.dumps(obj, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_metadata_as_dict(self, compact: bool = True):
        if compact:
            meta_result = dict()
            # merge metadata
            return {i["id"]: i["value"] for i in self.meta_data_items}
        else:
            return {
                "fields": self.meta_data_items,
                "flavor": self.flavor,
                "title": self.title,
            }

    def add_metadata_item(
        self, metadata_item: Union[EventMetadataItem, List[EventMetadataItem]]
    ):
        if isinstance(metadata_item, List):
            for i in metadata_item:
                if isinstance(i, dict):
                    self.meta_data_items.append(
                        EventMetadataItem(**self.rename_event_metadata_keys(i))
                    )
        else:
            if isinstance(metadata_item, dict):
                metadata_item = EventMetadataItem(
                    **self.rename_event_metadata_keys(metadata_item)
                )
            self.meta_data_items.append(metadata_item)

    @staticmethod
    def rename_event_metadata_keys(metadata_dict: dict):
        """
        Converts keys specified in camelCase form to snake_case form.
        :param metadata_dict:
        :return:
        """
        for key in metadata_dict.keys():
            metadata_dict[camel_case_to_snake_case(key)] = metadata_dict.pop(key)
        return metadata_dict


class PublicationMetadata:
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_id(self):
        return self.data["id"]

    def get_checksum(self):
        return self.data["checksum"]

    def get_flavor(self):
        return self.data["flavor"]

    def get_mediatype(self):
        return self.data["mediatype"]

    def get_size(self):
        return self.data["size"]

    def get_tags(self):
        return self.data["tags"]

    def get_url(self):
        return self.data["url"]


class Media:
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_id(self):
        return self.data["id"]

    def get_checksum(self):
        return self.data["checksum"]

    def get_bitrate(self):
        return self.data["bitrate"]

    def get_duration(self):
        return self.data["duration"]

    def get_description(self):
        return self.data["description"]

    def get_flavor(self):
        return self.data["flavor"]

    def get_framecount(self):
        return self.data["framecount"]

    def get_framerate(self):
        return self.data["framerate"]

    def has_audio(self):
        return self.data["has_audio"]

    def has_video(self):
        return self.data["has_video"]

    def get_height(self):
        return self.data["height"]

    def get_mediatype(self):
        return self.data["mediatype"]

    def get_size(self):
        return self.data["size"]

    def get_tags(self):
        return self.data["tags"]

    def get_url(self):
        return self.data["url"]

    def get_width(self):
        return self.data["width"]


class Attachment:
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_checksum(self):
        return self.data["checksum"]

    def get_flavor(self):
        return self.data["flavor"]

    def get_id(self):
        return self.data["id"]

    def get_mediatype(self):
        return self.data["mediatype"]

    def get_ref(self):
        return self.data["ref"]

    def get_size(self):
        return self.data["size"]

    def get_tags(self):
        return self.data["tags"]

    def get_url(self):
        return self.data["url"]


class Publication:
    def __init__(self, user, password, url, data):
        self.base_url = url
        self.user = user
        self.password = password
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_attachments(self, raw=False):
        if raw:
            return self.data["attachments"]
        else:
            attachments = list()
            for a in self.data["attachments"]:
                attachments.append(Attachment(a))
            return attachments

    def get_attachments_presentation_segment_preview(self) -> List[Attachment]:
        attachments = list()
        for a in self.get_attachments():
            if a.get_flavor() == "presentation/segment+preview":
                attachments.append(a)
        return attachments

    def get_attachments_presentation_timeline_preview(self) -> List[Attachment]:
        attachments = list()
        for a in self.get_attachments():
            if a.get_flavor() == "presentation/timeline+preview":
                attachments.append(a)
        return attachments

    def get_attachments_presentation_search_preview(self) -> List[Attachment]:
        attachments = list()
        for a in self.get_attachments():
            if a.get_flavor() == "presentation/search+preview":
                attachments.append(a)
        return attachments

    def get_attachments_presenter_player_preview(self) -> List[Attachment]:
        attachments = list()
        for a in self.get_attachments():
            if a.get_flavor() == "presenter/player+preview":
                attachments.append(a)
        return attachments

    def get_channel(self):
        return self.data["channel"]

    def get_id(self):
        return self.data["id"]

    def get_media(self, raw=False) -> List[Media]:
        if raw:
            return self.data["media"]
        else:
            media = list()
            for m in self.data["media"]:
                media.append(Media(m))
            return media

    def get_presentation_delivery_media(self) -> Optional[Media]:
        media = self.get_media()
        for m in media:
            if m.get_flavor() == "presentation/delivery":
                return m
        return None

    def get_mediatype(self):
        return self.data["mediatype"]

    def get_metadata(self, raw=False):
        if raw:
            return self.data["metadata"]
        else:
            metadata = list()
            for m in self.data["metadata"]:
                metadata.append(PublicationMetadata(m))
            return metadata

    def get_metadata_dc_episode(self) -> Optional[List[PublicationMetadata]]:
        for m in self.get_metadata():
            if m.get_flavor() == "dublincore/episode":
                return m
        return None

    def get_metadata_dc_series(self) -> Optional[List[PublicationMetadata]]:
        for m in self.get_metadata():
            if m.get_flavor() == "dublincore/series":
                return m
        return None

    def get_metadata_mpeg7_text(self) -> Optional[List[PublicationMetadata]]:
        for m in self.get_metadata():
            if m.get_flavor() == "mpeg-7/text":
                return m
        return None

    def get_url(self):
        return self.data["url"]



"""
def register_agent(address):
    params = [('address', config()['ui']['url']), ('state', status)]
        name = urlquote(config()['agent']['name'].encode('utf-8'), safe='')
        url = '%s/agents/%s' % (config()['service-capture.admin'][0], name)
        try:
            response = http_request(url, params).decode('utf-8')
            if response:
                logger.info(response)
        except pycurl.error as e:
            logger.warning('Could not set agent state to %s: %s' % (status, e))
"""

import re
from typing import Tuple, List, Optional

import xmltodict


def create_acl_rule_dict(role_name: str, action: str):
    d = {}
    _just_an_example = """
    <Rule RuleId="{ROLE_USER_ADMIN}_write_Permit" Effect="Permit">
		<Target>
			<Actions>
				<Action>
					<ActionMatch MatchId="urn:oasis:names:tc:xacml:1.0:function:string-equal">
						<AttributeValue DataType="http://www.w3.org/2001/XMLSchema#string">write</AttributeValue>
						<ActionAttributeDesignator AttributeId="urn:oasis:names:tc:xacml:1.0:action:action-id" DataType="http://www.w3.org/2001/XMLSchema#string"/>
					</ActionMatch>
				</Action>
			</Actions>
		</Target>
		<Condition>
			<Apply FunctionId="urn:oasis:names:tc:xacml:1.0:function:string-is-in">
				<AttributeValue DataType="http://www.w3.org/2001/XMLSchema#string">{ROLE_USER_ADMIN}</AttributeValue>
				<SubjectAttributeDesignator AttributeId="urn:oasis:names:tc:xacml:2.0:subject:role" DataType="http://www.w3.org/2001/XMLSchema#string"/>
			</Apply>
		</Condition>
	</Rule>
    """
    d["@RuleId"] = f"{role_name}_{action}_Permit"
    d["@Effect"] = "Permit"
    d["Target"] = {
        "Actions": {
            "Action": {
                "ActionMatch": {
                    "@MatchId": "urn:oasis:names:tc:xacml:1.0:function:string-equal",
                    "AttributeValue": {
                        "@DataType": "http://www.w3.org/2001/XMLSchema#string",
                        "#text": action,
                    },
                    "ActionAttributeDesignator": {
                        "@AttributeId": "urn:oasis:names:tc:xacml:1.0:action:action-id",
                        "@DataType": "http://www.w3.org/2001/XMLSchema#string",
                    },
                }
            }
        }
    }
    return d


def create_acl_xml(
    event_id: str,
    role_right_tuples: List[Tuple[str, str]],
    output: Optional[str] = None,
):
    # pylint: disable=line-too-long
    d = {
        "Policy": {
            "@PolicyId": event_id,
            "@Version": "2.0",
            "@RuleCombiningAlgId": "urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:permit-overrides",
            "@xmlns": "urn:oasis:names:tc:xacml:2.0:policy:schema:os",
            "Target": {
                "Resources": {
                    "Resource": {
                        "ResourceMatch": {
                            "@MatchId": "urn:oasis:names:tc:xacml:1.0:function:string-equal",
                            "AttributeValue": {
                                "@DataType": "http://www.w3.org/2001/XMLSchema#string",
                                "#text": event_id,
                            },
                            "ResourceAttributeDesignator": {
                                "@AttributeId": "urn:oasis:names:tc:xacml:1.0:resource:resource-id",
                                "@DataType": "http://www.w3.org/2001/XMLSchema#string",
                            },
                        }
                    }
                }
            },
        },
    }
    rules = []
    for role_right_tuple in role_right_tuples:
        rules.append(create_acl_rule_dict(role_right_tuple[0], role_right_tuple[1]))

    rules.append({"@RuleId": "DenyRule", "@Effect": "Deny"})
    d["Policy"]["Rule"] = rules
    if output is not None:
        with open(output, "a+", encoding="utf-8") as f:
            xmltodict.unparse(d, pretty=True, output=f)
    else:
        return xmltodict.unparse(d, pretty=True)


def pretty_print_resquest(req):
    """
    At this point it is completely built and ready
    to be fired; it is "prepared".

    However pay attention at the formatting used in
    this function because it is programmed to be pretty
    printed and may differ from the actual request.
    """
    print(
        "{}\n{}\n{}\n\n{}".format(
            "-----------START-----------",
            req.method + " " + req.url,
            "\n".join(f"{k}: {v}" for k, v in req.headers.items()),
            req.body,
        )
    )


first_cap_re = re.compile("(.)([A-Z][a-z]+)")
all_cap_re = re.compile("([a-z0-9])([A-Z])")


def camel_case_to_snake_case(string):
    """
    Converts a CamelCase string to the corresponding snake_case string
    :param string:
    :return:
    """
    s1 = first_cap_re.sub(r"\1_\2", string)
    return all_cap_re.sub(r"\1_\2", s1).lower()


def main():
    test_str = "HTTPCall and webLogin, readOnly"
    print(camel_case_to_snake_case(test_str))

    create_acl_xml(
        "event_id",
        [("ROLE_USER_ADMIN", "read"), ("ROLE_USER_ADMIN", "write")],
        output="/tmp/test",
    )


if __name__ == "__main__":
    main()



#  Copyright (c) 2019. Tobias Kurze
import os
from dotenv import load_dotenv
from loguru import logger

import pytest

from ocpy import setup_connection_config_from_yaml_file, get_connection_config
from ocpy.api import get_events_api_client, get_series_api_client
from ocpy.api.search import SearchApi
from ocpy.api.admin_ui.service import AdminServiceApi
from ocpy.api.sign import SigningApi
import tests

print("tada")

load_dotenv()  # alternative: to load local env


@pytest.fixture(scope="session", autouse=True)
def setup_oc_connection():
    # prepare something ahead of all tests
    print("setting up connection_config")
    setup_connection_config_from_yaml_file(os.path.abspath("tests/.test.creds.yml"))

    tests.EV_API = get_events_api_client()
    tests.SE_API = get_series_api_client()
    tests.SERVICE_API = AdminServiceApi()
    con_config = get_connection_config()
    if con_config is not None:
        tests.SIGN_API = SigningApi(**con_config)
    server = os.getenv("OC_PORTAL_SERVER")
    if server is None:
        logger.warning("OC_PORTAL_SERVER not set in environment")
    else:
        tests.SEARCH_API = SearchApi(
            user=os.getenv("OC_USER"), password=os.getenv("OC_PW"), server_url=server
        )


# setup_oc_connection()



import requests
from loguru import logger
from requests.auth import HTTPBasicAuth
import json
from pprint import pprint

from typing import List

from ocpy import OcPyRequestException
from ocpy.api.api_client import OpenCastBaseApiClient


class Theme:
    def __init__(self, user, password, url, data):
        self.base_url = url
        self.user = user
        self.password = password
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_bumper_file(self):
        return self.data["bumperFile"]

    def get_title_slide_background(self):
        return self.data["titleSlideBackground"]

    def get_watermark_file_url(self):
        return self.data["watermarkFileUrl"]

    def get_license_slide_background(self):
        return self.data["licenseSlideBackground"]

    def get_bumper_file_name(self):
        return self.data["bumperFileName"]

    def is_trailer_active(self):
        return self.data["trailerActive"]

    def get_default(self):
        return self.data["default"]

    def get_bumper_file_url(self):
        return self.data["bumperFileUrl"]

    def is_license_slide_active(self):
        return self.data["licenseSlideActive"]

    def is_watermark_active(self):
        return self.data["watermarkActive"]

    def get_trailer_file_url(self):
        return self.data["trailerFileUrl"]

    def is_title_slide_active(self):
        return self.data["titleSlideActive"]

    def get_trailer_file_name(self):
        return self.data["trailerFileName"]

    def get_id(self):
        return self.data["id"]

    def get_creator(self):
        return self.data["creator"]

    def is_bumper_active(self):
        return self.data["bumperActive"]

    def get_title_slide_metadata(self):
        return self.data["titleSlideMetadata"]

    def get_watermark_position(self):
        return self.data["watermarkPosition"]

    def get_creation_date(self):
        return self.data["creationDate"]

    def get_license_slide_description(self):
        return self.data["licenseSlideDescription"]

    def get_watermark_file_name(self):
        return self.data["watermarkFileName"]

    def get_watermark_file(self):
        return self.data["watermarkFile"]

    def get_name(self):
        return self.data["name"]

    def get_usage(self):
        url = self.base_url + "/usage.json"
        res = requests.get(url, auth=HTTPBasicAuth(self.user, self.password))
        if res.ok:
            return res.json()
        raise OcPyRequestException("could not get theme usage!", response=res)

    def delete(self):
        res = requests.delete(
            self.base_url, auth=HTTPBasicAuth(self.user, self.password)
        )
        if res.ok:
            return "ok"
        raise OcPyRequestException("could not delete theme!", response=res)


class ThemesApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/admin-ng/themes"

    def get_theme(self, theme_id: int, **kwargs) -> Theme:
        url = self.base_url + "/{}.json".format(theme_id)
        res = requests.get(url, auth=HTTPBasicAuth(self.user, self.password), **kwargs)
        if res.ok:
            theme = res.json()
            return Theme(
                self.user, self.password, self.base_url + "/{}".format(theme_id), theme
            )
        raise OcPyRequestException("could not get theme!", response=res)

    def get_themes(
        self, limit=100, offset=0, sort=None, themes_filter=None, **kwargs
    ) -> List[Theme]:
        """
        :param limit:
        :param offset:
        :param sort:
        :param themes_filter:
        :param kwargs:
        :return: List[Event]
        """
        parameters = {"limit": limit, "offset": offset, "sort": sort}
        if themes_filter:
            if isinstance(themes_filter, ThemesApi.Filter):
                parameters["filter"] = themes_filter.get_filter_string()
            else:
                parameters["filter"] = themes_filter
        results = []
        res = requests.get(
            self.base_url + "/themes.json",
            auth=HTTPBasicAuth(self.user, self.password),
            params=parameters,
            **kwargs
        )
        if res.ok:
            for theme in res.json()["results"]:
                results.append(
                    Theme(
                        self.user,
                        self.password,
                        self.base_url + "/{}".format(theme["id"]),
                        theme,
                    )
                )
        else:
            if res.status_code == 500:
                logger.warning(
                    "Opencast responded with an internal server error (500)."
                )
            else:
                logger.error(
                    "Request to Opencast resulted in an error (code: {}): {}".format(
                        res.status_code, res.content
                    )
                )
            raise OcPyRequestException(res.content.decode("utf-8"), response=res)
        return results

    def get_all_themes(self) -> List[Theme]:
        result = []
        while True:
            res = self.get_themes(limit=100, offset=len(result))
            if res is None or len(res) <= 0:
                break
            result.extend(res)
        return result

    def create_theme(
        self,
        name: str,
        default: bool = False,
        description: str = None,
        bumper_active: bool = None,
        trailer_active: bool = None,
        title_slide_active: bool = None,
        license_slide_active: bool = None,
        watermark_active: bool = None,
        bumper_file: str = None,
        trailer_file: str = None,
        watermark_file: str = None,
        title_slide_background: str = None,
        license_slide_background: str = None,
        title_slide_metadata: str = None,
        license_slide_description: str = None,
        watermark_position: str = None,
        **kwargs
    ) -> Theme:
        data = {
            "name": name,
            "default": default,
            "description": description,
            "bumperActive": bumper_active,
            "trailerActive": trailer_active,
            "titleSlideActive": title_slide_active,
            "licenseSlideActive": license_slide_active,
            "watermarkActive": watermark_active,
            "bumperFile": bumper_file,
            "trailerFile": trailer_file,
            "watermarkFile": watermark_file,
            "titleSlideBackground": title_slide_background,
            "licenseSlideBackground": license_slide_background,
            "titleSlideMetadata": title_slide_metadata,
            "licenseSlideDescription": license_slide_description,
            "watermarkPosition": watermark_position,
        }
        clean_data = {key: data[key] for key in data if data[key] is not None}
        res = requests.post(
            self.base_url,
            data=clean_data,
            auth=HTTPBasicAuth(self.user, self.password),
            **kwargs
        )
        if res.ok:
            theme = res.json()
            return Theme(
                self.user,
                self.password,
                self.base_url + "/{}".format(theme["id"]),
                theme,
            )
        raise OcPyRequestException("could not delete theme!", response=res)

    def update_theme(
        self,
        theme_id: int,
        name: str,
        default: bool = False,
        description: str = None,
        bumper_active: bool = None,
        trailer_active: bool = None,
        title_slide_active: bool = None,
        license_slide_active: bool = None,
        watermark_active: bool = None,
        bumper_file: str = None,
        trailer_file: str = None,
        watermark_file: str = None,
        title_slide_background: str = None,
        license_slide_background: str = None,
        title_slide_metadata: str = None,
        license_slide_description: str = None,
        watermark_position: str = None,
        **kwargs
    ) -> Theme:
        url = self.base_url + "/{}".format(theme_id)
        data = {
            "name": name,
            "default": default,
            "description": description,
            "bumperActive": bumper_active,
            "trailerActive": trailer_active,
            "titleSlideActive": title_slide_active,
            "licenseSlideActive": license_slide_active,
            "watermarkActive": watermark_active,
            "bumperFile": bumper_file,
            "trailerFile": trailer_file,
            "watermarkFile": watermark_file,
            "titleSlideBackground": title_slide_background,
            "licenseSlideBackground": license_slide_background,
            "titleSlideMetadata": title_slide_metadata,
            "licenseSlideDescription": license_slide_description,
            "watermarkPosition": watermark_position,
        }
        clean_data = {key: data[key] for key in data if data[key] is not None}
        res = requests.put(
            url, data=clean_data, auth=HTTPBasicAuth(self.user, self.password), **kwargs
        )
        if res.ok:
            theme = res.json()
            return Theme(
                self.user,
                self.password,
                self.base_url + "/{}".format(theme["id"]),
                theme,
            )
        raise OcPyRequestException("could not delete theme!", response=res)

    def delete_theme(self, theme_id: int, **kwargs):
        url = self.base_url + "/{}".format(theme_id)
        res = requests.delete(
            url, auth=HTTPBasicAuth(self.user, self.password), **kwargs
        )
        if res.ok:
            return "ok"
        raise OcPyRequestException("could not delete theme!", response=res)

    class Filter:
        def __init__(self):
            self.filter_string = None

        def update_filter_string(self, update_string):
            if self.filter_string is None:
                self.filter_string = update_string
            else:
                self.filter_string += "," + update_string

        def set_name_filter(self, name):
            self.update_filter_string("name:{}".format(name))
            return self

        def set_creator_filter(self, creator):
            self.update_filter_string("creator:{}".format(creator))
            return self

        def get_filter_string(self):
            return self.filter_string


def main():
    api = ThemesApi()
    # pprint(api.get_theme(36351).get_usage())
    # pprint(api.create_theme(name="tolles tests theme"))

    themes = api.get_themes()
    pprint(themes)
    # print("delete tests:")
    # pprint(themes[-1].delete())
    # pprint(api.get_all_themes())


if __name__ == "__main__":
    main()



import json
from loguru import logger
from pprint import pprint
from typing import List, Optional

import xmltodict

from ocpy.model.publication import PublicationList
from ocpy.model.track import TrackList


class MediaPackage:
    def __init__(self, xml_data):
        self.xml_data = xml_data
        self.data = xmltodict.parse(xml_data)
        self.data = self.data["mediapackage"] if "mediapackage" in self.data else self.data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4,
                          separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_xml(self):
        return self.xml_data

    def get_id(self) -> str:
        return self.data["@id"]

    def get_start(self) -> str:
        return self.data["@start"]

    def get_xmlns(self) -> str:
        return self.data["@xmlns"]

    def get_attachments(self) -> str:
        return self.data["attachments"]

    def get_media(self) -> Optional[TrackList]:
        if self.data["media"] is None:
            logger.info(f"No media found in mediapackage {self.get_id()}")
            return TrackList([])
        tracks = self.data["media"].get("track",None)
        logger.trace(f"tracks: {tracks}")
        if tracks is None:
            return TrackList([])
        return TrackList(tracks)

    def get_metadata(self) -> str:
        return self.data["metadata"]

    def get_publications(self) -> PublicationList:
        publications = self.data["publications"].get("publication")
        return PublicationList(publications)



# -*- coding: utf-8 -*-
"""Opencast admin_ui Resource API.

Base implementation of opencast admin_ui resource api.
Please refer to und use external resource API functions.

"""

import json
import requests
from loguru import logger
from requests.auth import HTTPBasicAuth

from ocpy import OcPyRequestException
from ocpy.api.api_client import OpenCastBaseApiClient


class ResourcesApi(OpenCastBaseApiClient):
    """admin_ui.EventAPI class."""

    def __init__(self, user=None, password=None, server_url=None):
        """Create new event API instance.

        Args:
            user: Opencast username
            password: Opencast password
            server_url: Base URL of Opencast Admin Node
        """
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/admin-ng/resources"

    def get_event_upload_options(self, **kwargs) -> str:
        """Return metadata required to fill metadata.

        Return:
            str: metadata
        Raises:
             OcPyRequestException: if metadata could not be retrieved
        """
        url = self.base_url + "/eventUploadAssetOptions.json"
        res = requests.get(
            url, auth=HTTPBasicAuth(self.user, self.password), timeout=10, **kwargs
        )
        if res.ok:
            fixed_res = dict()
            for key, value in res.json().items():
                logger.debug(f"{key}: {value}")
                try:
                    fixed_res[key] = json.loads(value)
                except json.decoder.JSONDecodeError:
                    fixed_res[key] = value
            return fixed_res
        raise OcPyRequestException("could not get metadata tags!", response=res)



#!/usr/bin/env python3
import tempfile
import xml
import os
from loguru import logger
from random import choice

import requests
from requests import Request, Session
from requests.auth import HTTPDigestAuth, HTTPBasicAuth
import json
import xmltodict
from pprint import pprint

from typing import List, Tuple, Optional
import xml.etree.ElementTree as ElemTree

from ocpy.model.mediapackage import MediaPackage
from ocpy.api.api_client import OpenCastBaseApiClient, \
    OpenCastDigestBaseApiClient, OpenCastApiService
from ocpy.utils import create_acl_xml


class IngestApi(OpenCastApiService, OpenCastBaseApiClient,
                OpenCastDigestBaseApiClient):
    def __init__(
            self,
            service_url,
            user=None,
            password=None,
            digest_user=None,
            digest_password=None,
            use_digest_auth=False,
            **kwargs
    ):
        OpenCastApiService.__init__(self, service_url)
        OpenCastBaseApiClient.__init__(self, user, password)
        OpenCastDigestBaseApiClient.__init__(self, digest_user,
                                             digest_password, optional=True)
        self.use_digest_auth = self.use_digest_auth

        self.session = Session()
        if self.use_digest_auth:
            self.session.auth = HTTPDigestAuth(self.digest_user,
                                               self.digest_password)
            self.session.headers.update({"X-Requested-Auth": "Digest"})
        else:
            self.session.auth = HTTPBasicAuth(self.user, self.password)

    def create_empty_media_package(self, **kwargs) -> MediaPackage:
        url = self.base_url + "/createMediaPackage"
        res = self.session.get(
            url,
            **kwargs
        )
        if res.ok:
            try:
                return MediaPackage(res.text)
            except xml.parsers.expat.ExpatError:
                raise Exception(
                    "No valid XML response (probably authentication failure)!"
                )
        raise Exception(
            "Could not create empty media_package! ({})".format(
                res.status_code)
        )

    def add_attachment(
            self, media_package: MediaPackage, flavor: str, file: str, **kwargs
    ):
        url = self.base_url + "/addAttachment"
        data = {"flavor": flavor, "mediaPackage": media_package.get_xml()}

        with open(os.path.expanduser(file), "rb") as f:
            files = {"file": f}
            request = Request(
                method="POST",
                url=url,
                data=data,
                files=files,
                **kwargs
            )
            res = self.session.send(self.session.prepare_request(request))
            print(res.url)
            if res.ok:
                return MediaPackage(res.content)
            print(res.content)
            print(res.status_code)
            raise Exception("Could not add attachment!")

    def add_track(
            self,
            media_package: MediaPackage,
            flavor: str = "presentation/source",
            file: Optional[str] = None,
            url: Optional[str] = None,
    ):
        print("adding track")
        if (file is None and url is None) or (
                file is not None and url is not None):
            raise ValueError(
                "exactly one of file and url has to be specified!")
        track_url = self.base_url + "/addTrack"
        if url is not None:
            data = {
                "url": url,
                "flavor": flavor,
                "mediaPackage": media_package.get_xml(),
            }
            request = Request(
                method="POST",
                url=track_url,
                data=data,
            )
            res = self.session.send(self.session.prepare_request(request))
        elif file is not None:
            with open(os.path.expanduser(file), "rb") as f:
                files = {"file": f}

                data = {"flavor": flavor,
                        "mediaPackage": media_package.get_xml()}
                request = Request(
                    method="POST",
                    url=track_url,
                    data=data,
                    files=files,
                )
                res = self.session.send(self.session.prepare_request(request))

        print(res.url)
        if res.ok:
            return MediaPackage(res.content)
        print(res.content)
        print(res.status_code)
        raise Exception("Could not add track!")

    def add_media_package(
            self, files: List[Tuple[str, str]], workflow_id: str = None,
            **kwargs
    ):
        """
        :param files:
        :param workflow_id:
        :param kwargs:
        :return:
        """
        if "title" not in kwargs:
            raise ValueError("Title must be specified!")
        url = self.base_url + "/addMediaPackage"
        if workflow_id is not None:
            url += "/" + workflow_id
        form_data = []
        for k in kwargs:
            form_data.append((k, (None, kwargs[k])))
        if isinstance(files, str):
            files = [files]
        if len(files) == 1 and isinstance(files[0], str):
            form_data.append(("flavor", (None, "presentation/source")))
            form_data.append(("file", open(files[0], "rb")))
        else:
            for file in files:
                if not isinstance(file, tuple):
                    raise ValueError(
                        "files must be a list of tuples in the form: (flavor, file)"
                    )
                form_data.append(("flavor", (None, file[0])))
                form_data.append(("file", open(file[1], "rb")))

        for k, v in kwargs.items():
            form_data.append((k, v))

        request = Request(
            method="POST",
            url=url,
            files=form_data,
        )
        res = self.session.send(self.session.prepare_request(request))
        if res.ok:
            return res.text
        else:
            logger.critical("Could not add media package to Opencast!")
            logger.critical("Status code: " + str(res.status_code))
            print(request.url)
            print(request.headers)

        raise Exception("Could not add media package to Opencast!")

    def add_dc_catalog(
            self,
            media_package: MediaPackage,
            dublin_core_xml: str,
            flavor="dublincore/episode",
    ):
        url = self.base_url + "/addDCCatalog"
        data = {"mediaPackage": media_package.get_xml(),
                "dublinCore": dublin_core_xml}
        if flavor is not None:
            data["flavor"] = flavor

        request = Request(
            method="POST",
            url=url,
            data=data,
        )
        res = self.session.send(self.session.prepare_request(request))
        if res.ok:
            return MediaPackage(res.content)
        logger.critical("Could not add dc catalog")
        logger.critical("Status code: " + str(res.status_code))
        raise Exception("Could not add dc catalog!")

    def add_metadata(
            self,
            media_package: MediaPackage,
            flavor="dublincore/episode",
            root_element_name="dublincore",
            abstract: str = None,
            accessRights: str = None,
            available: str = None,
            contributor: str = None,
            coverage: str = None,
            created: str = None,
            creator: str = None,
            date: str = None,
            description: str = None,
            extent: str = None,
            format: str = None,
            identifier: str = None,
            isPartOf: str = None,
            isReferencedBy: str = None,
            isReplacedBy: str = None,
            language: str = None,
            license: str = None,
            publisher: str = None,
            relation: str = None,
            replaces: str = None,
            rights: str = None,
            rightsHolder: str = None,
            source: str = None,
            spatial: str = None,
            subject: str = None,
            temporal: str = None,
            title: str = None,
            type: str = None,
            **kwargs
    ):
        """

        :param media_package:
        :param flavor:
        :param abstract:
        :param accessRights:
        :param available:
        :param contributor:
        :param coverage:
        :param created:
        :param creator:
        :param date:
        :param description:
        :param extent: length of media file (in ISO8601 format) can perhaps left empty
        :param format:
        :param identifier:
        :param isPartOf:
        :param isReferencedBy:
        :param isReplacedBy:
        :param language:
        :param license:
        :param publisher:
        :param relation:
        :param replaces:
        :param rights:
        :param rightsHolder:
        :param source:
        :param spatial:
        :param subject:
        :param temporal:
        :param title:
        :param type:
        :return:
        """

        # TODO: implement other metadata fields

        root = ElemTree.Element(root_element_name)
        root.set('xmlns', 'http://www.opencastproject.org/xsd/1.0/dublincore/')
        root.set('xmlns:dcterms', 'http://purl.org/dc/terms/')
        root.set('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance')
        if title is not None:
            ElemTree.SubElement(root, 'dcterms:title').text = title
        if description is not None:
            ElemTree.SubElement(root, 'dcterms:description').text = description
        if isPartOf is not None:
            ElemTree.SubElement(root, 'dcterms:isPartOf').text = isPartOf
        if source is not None:
            ElemTree.SubElement(root, 'dcterms:source').text = source
        if creator is not None:
            ElemTree.SubElement(root, 'dcterms:creator').text = creator
        if spatial is not None:
            ElemTree.SubElement(root, 'dcterms:spatial').text = spatial
        if rightsHolder is not None:
            ElemTree.SubElement(root,
                                'dcterms:rightsHolder').text = rightsHolder
        if temporal is not None:
            ElemTree.SubElement(root, 'dcterms:temporal').set('xsi:type',
                                                              'dcterms:Period')
            ElemTree.SubElement(root, 'dcterms:temporal').text = temporal

        temporal_str = "start={start}; end={end}; scheme=W3C-DTF;"

        """
         $start_end_string_iso = (new ilDateTime(
            strtotime($this->metadata->getField('startDate')->getValue() . ' ' . $this->metadata->getField('startTime')->getValue()),
            IL_CAL_UNIX)
        )->get(IL_CAL_FKT_DATE, 'Y-m-d\TH:i:s.u\Z');
        $xml_writer->xmlElement('dcterms:temporal', [
            'xsi:type' => 'dcterms:Period'
        ], 'start=' . $start_end_string_iso . '; ' . 'end=' . $start_end_string_iso . '; scheme=W3C-DTF;');
        """

        # ElemTree.SubElement(root, 'dcterms:created').text = created
        """
        $xml_writer->xmlElement(
            'dcterms:created',
            [],
            (new ilDateTime(time(), IL_CAL_UNIX))
                ->get(IL_CAL_FKT_DATE, 'Y-m-d\TH:i:s.u\Z', 'UTC')
        );
        """

        for k, v in kwargs.items():
            ElemTree.SubElement(root, f"dcterms:{k}").text = v

        dublin_core_xml = ElemTree.tostring(root, method='xml').decode()

        return self.add_dc_catalog(media_package,
                                   dublin_core_xml=dublin_core_xml,
                                   flavor=flavor)

    def ingest(
            self,
            media_package: MediaPackage,
            workflow_definition_id: str,
            workflow_instance_id: str = None,
            # workflow_parameters: List[str] = [],
            **kwargs
    ):
        url = self.base_url + "/ingest"
        if workflow_instance_id is None:
            url += "/" + workflow_definition_id
            data = {
                **kwargs,
                "mediaPackage": media_package.get_xml()
            }
        else:
            data = {
                **kwargs,
                "mediaPackage": media_package.get_xml(),
                "workflowDefinitionId": workflow_definition_id,
                "workflowInstanceId": workflow_instance_id
            }

        request = Request(
            method="POST",
            url=url,
            data=data
        )
        res = self.session.send(self.session.prepare_request(request))

        print(res.url)
        if res.ok:
            return res.content
        print(res.content)
        print(res.status_code)
        raise Exception("Could not ingest media package!")

    def add_acl_attachment(self, media_package: MediaPackage, event_id: str, role_right_tuples: List[Tuple[str, str]],
                           **kwargs):
        flavor = "security/xacml+episode"
        """
         $plupload = new Plupload();
        $tmp_name = uniqid('tmp');
        file_put_contents($plupload->getTargetDir() . '/' . $tmp_name, (new ACLtoXML($event->getAcl()))->getXML());
        $upload_file = new xoctUploadFile();
        $upload_file->setFileSize(filesize($plupload->getTargetDir() . '/' . $tmp_name));
        $upload_file->setPostVar('attachment');
        $upload_file->setTitle('attachment');
        $upload_file->setTmpName($tmp_name);
        return $upload_file;
        :return:
        """

        with tempfile.TemporaryDirectory() as tmp:
            path = os.path.join(tmp, 'security-policy-episode.xml')
            create_acl_xml(event_id, role_right_tuples, output=path)

            return self.add_attachment(media_package, flavor, file=path)


def main():
    from ocpy.api.service import ServiceApi, ServiceType

    user = "api_user"
    password = "..."
    digest_user = "opencast_system_account"
    digest_password = "..."
    s_api = ServiceApi(
        server_url="https://opencast-qa.bibliothek.kit.edu",
        user=user,
        password=password,
    )
    # s_api = ServiceApi(server_url="http://localhost:8080", user="admin", password="opencast")
    # s_api = ServiceApi()
    ingest_service = choice(s_api.get_available(ServiceType.ingest))

    api = IngestApi(
        ingest_service.get_url(),
        user=user,
        password=password,
        digest_user=digest_user,
        digest_password=digest_password,
    )
    print(api)
    print("Create empty media_package:")
    m_p = api.create_empty_media_package()
    print(m_p)
    print(m_p.get_xml())
    m_p_xml = m_p.get_xml()
    m_p = api.add_attachment(m_p, "audio/123", "/home/tobias/Music/Poison.mp3")
    print(m_p)
    # TODO: check if it is necessary to update the local mediapackage with a function result?!
    # api.add_track(m_p, url="https://sample-videos.com/video123/mp4/720/big_buck_bunny_720p_2mb.mp4")
    api.add_track(m_p, file="/home/tobias/Videos/vid2.mp4")
    # m_p = api.add_attachment(m_p, "presentation/source", "/home/tobias/Videos/vid2.mp4")
    print(m_p)
    # m_p2 = api.add_media_package("/home/tobias/Videos/vid2.mp4", "fast", title="tests ingest per API")
    # print(m_p2)
    # api.ingest(m_p, "fast")

    exit()

    # res = api.add_attachment("presentation/source", m_p, "/tmp/swtag.log")
    if os.path.exists("/Users/tobias/Movies"):  # on MAC
        res = api.add_media_package(
            [
                (
                    "presentation/source",
                    "/Users/tobias/Movies/SampleVideo_1280x720_5mb.mp4",
                )
            ],
            "fast",
            title="Test media package (MAC)",
        )
    else:  # on Linux
        # res = api.add_media_package(["/home/tobias/Videos/vid2.mp4"],
        #                            "fast", title="Test (Linux, two-files)")
        #        res = api.add_media_package([("presentation/source", "/home/tobias/Videos/vid2.mp4")],
        #                                    "fast", title="Test (Linux, two-files)")
        res = api.add_media_package(
            [
                ("presentation/source", "/home/tobias/Videos/vid1.mp4"),
                ("presenter/source", "/home/tobias/Videos/vid2.mp4"),
            ],
            "fast",
            title="New vid tests (Linux, two-files)",
        )
    print(res)


if __name__ == "__main__":
    main()



import requests
from pprint import pprint
from requests.auth import HTTPBasicAuth

from ocpy import OcPyRequestException
from ocpy.api.api_client import OpenCastBaseApiClient


class OrgApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **_kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/org"

    def get_all_orgs(self, **kwargs):
        res = requests.get(
            self.base_url + "/all.json",
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.json()
        raise OcPyRequestException("could not get event!", response=res)

    def get_org(self, id: str, **kwargs):
        res = requests.get(
            self.base_url + "/{}.json".format(id),
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.json()
        raise OcPyRequestException("could not get event!", response=res)


def main():
    api = OrgApi()
    pprint(api.get_all_orgs())
    pprint(api.get_org("mh_default_org"))


if __name__ == "__main__":
    main()



from pprint import pformat
from loguru import logger
import unittest
import pytest

from ocpy.api.admin_ui.resources import ResourcesApi


@pytest.mark.usefixtures("setup_oc_connection")
class TestResourceClass(unittest.TestCase):
    def test_get_services(self):
        admin_ev_api = ResourcesApi()
        event_upload_options = admin_ev_api.get_event_upload_options()
        logger.debug(pformat(event_upload_options))
        self.assertIsNotNone(event_upload_options)
        self.assertIsInstance(event_upload_options, dict)



# -*- coding: utf-8 -*-
"""Opencast admin_ui EventAPI.

Base implementation of opencast admin_ui event api.
Please refer to und use external event API functions.

"""
import json
import mimetypes
import os
from pprint import pformat
from typing import Dict, Optional
from loguru import logger
import requests
from requests.auth import HTTPBasicAuth

from ocpy import OcPyRequestException
from ocpy.api.admin_ui.resources import ResourcesApi
from ocpy.api.api_client import OpenCastBaseApiClient


class EventApi(OpenCastBaseApiClient):
    """admin_ui.EventAPI class."""

    def __init__(self, user=None, password=None, server_url=None, **_kwargs):
        """Create new event API instance.

        Args:
            user: Opencast username
            password: Opencast password
            server_url: Base URL of Opencast Admin Node
        """
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/admin-ng/event"
        self.session = requests.Session()
        self.session.auth = HTTPBasicAuth(self.user, self.password)

    def get_new_metadata(self, timeout=10, **kwargs) -> str:
        """Return metadata required to fill metadata.

        Return:
            str: metadata
        Raises:
             OcPyRequestException: if metadata could not be retrieved
        """
        url = self.base_url + "/new/metadata"
        res = requests.get(
            url, timeout=timeout, auth=HTTPBasicAuth(self.user, self.password), **kwargs
        )
        if res.ok:
            return res.json()
        raise OcPyRequestException("could not get metadata tags!", response=res)

    def get_assets(self, event_id: str, timeout=10):
        """Return assets of a given event.

        Params:
            event_id: id of the event
        Return:
            str: metadata
        Raises:
             OcPyRequestException: if metadata could not be retrieved
        """
        url = f"{self.base_url}/{event_id}/asset/assets.json"
        res = requests.get(
            url, timeout=timeout, auth=HTTPBasicAuth(self.user, self.password)
        )
        if res.ok:
            return res.json()
        raise OcPyRequestException("could not get assets!", response=res)

    def get_attachments(self, event_id: str, timeout=10):
        """Return attachments of a given event.

        Params:
            event_id: id of the event
        Return:
            str: metadata
        Raises:
             OcPyRequestException: if metadata could not be retrieved
        """
        url = f"{self.base_url}/{event_id}/asset/attachment/attachments.json"
        res = requests.get(
            url, timeout=timeout, auth=HTTPBasicAuth(self.user, self.password)
        )
        if res.ok:
            return res.json()
        raise OcPyRequestException("could not get attachments!", response=res)

    def _available_event_upload_options_to_list(
        self, available_event_upload_options: Dict[str, Dict[str, str]]
    ):
        workflow_id = None
        option_list = []
        for o in available_event_upload_options.keys():
            try:
                option_list.append(
                    # {**available_event_upload_options[o], "title": o}
                    {
                        **available_event_upload_options[o],
                        "title": o,
                        "showsAs": "uploadAsset",
                    }
                )
            except TypeError:
                logger.debug(o)
                workflow_id = available_event_upload_options[o]

        return option_list, workflow_id

    def get_mime_type(self, file):
        """Return mime type of a given file (by guessing)."""
        mime_type = mimetypes.guess_type(file)[0]
        if not mime_type:
            mime_type = "application/octet-stream"
        return mime_type

    def post_attachment(
        self,
        event_id,
        attachment_name,
        file,
        publish_uploaded_assets_wf=None,
        available_event_upload_options: Optional[Dict] = None,
    ):
        """Adds an attachment to an event.
        If there is already an attachment with the same name, it will be overwritten.

        Args:
            event_id (string): The ID (UUID) of the event to which the attachment should be added.
            attachment_name (string): Name of the attachment (as configured in Opencast).
            file (string): Path to the file to be uploaded.
            publish_uploaded_assets_wf (string, optional):
                Name of the workflow to run after attachment was added. Defaults to None.
            available_event_upload_options (Optional[Dict], optional):
                Dict of available asset upload options. Defaults to None.

        Raises:
            OcPyRequestException: If file does not exist
            OcPyRequestException: If attachment_name does not exist / is not configured
            OcPyRequestException: If request failed

        Returns:
            string: Workflow ID
        """
        url = f"{self.base_url}/{event_id}/assets"

        if not os.path.isfile(file):
            raise OcPyRequestException(f"File {file} does not exist!")

        workflow_id = None
        if not available_event_upload_options:
            r_api = ResourcesApi()
            (
                available_event_upload_options,
                workflow_id,
            ) = self._available_event_upload_options_to_list(
                r_api.get_event_upload_options()
            )

        if publish_uploaded_assets_wf is None:
            if workflow_id:
                publish_uploaded_assets_wf = workflow_id
            else:
                publish_uploaded_assets_wf = "publish-uploaded-assets"

        try:
            event_upload_option = [
                available_event_upload_option
                for available_event_upload_option in available_event_upload_options
                if available_event_upload_option["id"] == attachment_name
            ].pop()
        except IndexError:
            event_upload_option = None
        if not event_upload_option:
            raise OcPyRequestException(
                f"Attachment {attachment_name} does not exist / is not configured!"
            )

        data = {
            "assets": {"options": available_event_upload_options},
            "processing": {
                "workflow": publish_uploaded_assets_wf,
                "configuration": {
                    "downloadSourceflavorsExist": "true",
                    "download-source-flavors": f"{event_upload_option['flavorType']}/{event_upload_option['flavorSubType']}",  # pylint: disable=line-too-long
                },
            },
        }
        logger.debug(pformat(data))

        with open(os.path.expanduser(file), "rb") as f:
            file_name = os.path.basename(file)
            mime_type = self.get_mime_type(file)
            files = {
                f"{attachment_name}.0": (file_name, f, mime_type)
            }  # a mime type (content type) is required!
            data = json.dumps(data)
            request = self.session.prepare_request(
                requests.Request(
                    method="POST", url=url, data={"metadata": data}, files=files
                )
            )
            # pretty_print_POST(request)
            res = self.session.send(request)
            if res.ok:
                logger.debug(f"{res.status_code}, {res.text}")
                return res.text
            logger.error(f"{res.status_code}, {res.text}")
            raise OcPyRequestException("could not post attachment!", response=res)

    def post_subtitles(
        self, event_id, file, language_code="de", publish_uploaded_assets_wf=None
    ):
        """Adds subtitles attachment to an event.
        If there is already a subtitles attachment for the language, it will be overwritten.

        Args:
            event_id (string): ID of the event to which the attachment should be added.
            file (string): Path to the file to be uploaded.
            language_code (str, optional): Language code of the subtitles. Defaults to "de".
            publish_uploaded_assets_wf (string, optional):
                Name of the workflow to run after attachment was added. Defaults to None.

        Raises:
            OcPyRequestException: Upload option for subtitles not found

        Returns:
            string: WF ID
        """
        r_api = ResourcesApi()
        (
            available_event_upload_options,
            workflow_id,
        ) = self._available_event_upload_options_to_list(
            r_api.get_event_upload_options()
        )
        # following line for OC < 15
        possible_upload_options = [
            option
            for option in available_event_upload_options
            if ("captions" in option["id"] or "track_subtitles_option" in option["id"])
            and (
                f"+{language_code}" in option["flavorSubType"]
                or f"lang:{language_code}" in option["tags"]
            )
        ]
        if len(possible_upload_options) == 0:
            logger.error(
                f"Only found these upload options: {pformat(available_event_upload_options)}"
            )
            raise OcPyRequestException("No upload option for subtitles found!")

        possible_upload_option = possible_upload_options[
            0
        ]  # pick the first (and hopefully only) option
        if not publish_uploaded_assets_wf:
            publish_uploaded_assets_wf = workflow_id
        return self.post_attachment(
            event_id, possible_upload_option["id"], file, publish_uploaded_assets_wf
        )



"""Module for OC Event API (external API)"""

import os
import json
from typing import Generator, List, Optional, Union

import pendulum
import requests
from loguru import logger
from pendulum.parsing import ParserError
from requests.auth import HTTPBasicAuth

from ocpy.api.api_client import OpenCastBaseApiClient
from ocpy import OcPyRequestException
from ocpy.model.event_models import Publication
from ocpy.model.publication import PublicationList
from ocpy.model.acl import ACL, Action
from ocpy.model.scheduling import Scheduling


class Event:
    """Class for Opencast events"""

    def __init__(self, user, password, url, data):
        self.base_url = url
        self.user = user
        self.password = password
        self.data: dict = data
        self._augment_data()

    def __str__(self):
        return json.dumps(
            self.data,
            sort_keys=True,
            indent=4,
            separators=(",", ": "),
            default=str,
        )

    def __repr__(self):
        return self.__str__()

    def _augment_data(self):
        if "created" in self.data and self.data["created"]:
            try:
                if isinstance(self.data["created"], str):
                    self.data["created"] = pendulum.parse(self.data["created"])
            except ParserError:
                pass
            except ValueError:
                pass
        if "start" in self.data and self.data["start"]:
            try:
                if isinstance(self.data["start"], str):
                    self.data["start"] = pendulum.parse(self.data["start"])
            except ParserError:
                pass
            except ValueError:
                pass
        if "duration" in self.data and self.data["duration"]:
            try:
                self.data["duration"] = int(self.data["duration"])
            except ValueError:
                pass

    def get_archive_version(self):
        return self.data.get("archive_version")

    def get_contributor(self):
        return self.data.get("contributor")

    def get_created(self):
        return self.data.get("created")

    def get_creator(self):
        return self.data.get("creator")

    def get_description(self):
        return self.data.get("description")

    def get_duration(self):
        return self.data.get("duration")

    def has_previews(self):
        return self.data.get("has_previews")

    def get_identifier(self):
        return self.data.get("identifier")

    def get_is_part_of(self):
        return self.data.get("is_part_of", None)

    def get_series_name(self):
        return self.data.get("series", None)

    def get_location(self):
        return self.data.get("location")

    def get_presenter(self):
        return self.data.get("presenter")

    def get_processing_state(self):
        return self.data.get("processing_state")

    def get_publication_status(self):
        return self.data.get("publication_status")

    def get_status(self):
        return self.data.get("status")

    def get_start(self):
        return self.data.get("start")

    def get_subjects(self):
        return self.data.get("subjects")

    def get_title(self):
        return self.data.get("title")

    def get_acl(self, **kwargs) -> Optional[List[ACL]]:
        res = self.data.get("acl", None)
        if res is None:
            url = self.base_url + "/acl"
            res = requests.get(
                url,
                timeout=kwargs.pop("timeout", 30),
                auth=HTTPBasicAuth(self.user, self.password),
            )
            if res.ok:
                res = res.json()
            else:
                return None
        acls = []
        for a in res:
            acls.append(ACL(a["allow"], a["role"], Action(a["action"])))
        return acls

    def set_acl(self, acl_list, **kwargs):
        url = self.base_url + "/acl"
        if isinstance(acl_list, List):
            acls = []
            for a in acl_list:
                if isinstance(a, ACL):
                    acls.append(a.get_acl_dict())
                else:
                    acls.append(a)
            acl_list = acls
        if isinstance(acl_list, ACL):
            acl_list = [acl_list.get_acl_dict()]

        data = {
            "acl": json.dumps(acl_list),
        }
        logger.debug(f"set acl data: {data}")
        res = requests.put(
            url,
            timeout=kwargs.pop("timeout", 30),
            data=data,
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if res.ok:
            return "ok"
        raise OcPyRequestException(
            "could not set ACL! ("
            + res.text
            + " ["
            + str(res.status_code)
            + "])"
            + "acl: "
            + json.dumps(acl_list)
        )

    def add_to_acl(self, acl: Union[ACL, List[ACL]]):
        logger.debug(f"adding to acl: {acl}")
        actual_acl = self.get_acl()
        if not isinstance(acl, List):
            acl = [acl]
        for n_a in acl:
            found = False
            for a_a in actual_acl:
                if a_a == n_a:
                    found = True
                    break
            if not found:
                actual_acl.append(n_a)

        return self.set_acl(actual_acl)

    def remove_from_acl(self, acl: ACL):
        actual_acl = self.get_acl()

        if not isinstance(acl, List):
            acl = [acl]
        for aa in actual_acl:
            for na in acl:
                if aa == na:
                    actual_acl.remove(na)
        return self.set_acl(actual_acl)

    def get_metadata(self, raw=False, force_update=False, **kwargs):
        if "metadata" in self.data and not force_update:
            return self.data["metadata"]
        url = self.base_url + "/metadata"
        res = requests.get(
            url,
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if res.ok:
            if raw:
                return res.json()
            meta_result = dict()
            for r in res.json():
                # merge metadata
                meta_result = {
                    **meta_result,
                    **{m["id"]: m["value"] for m in r["fields"] if "value" in m},
                }
            return meta_result
        return None

    def delete_metadata(self, **kwargs):
        url = self.base_url + "/metadata"
        params = {"type": "dublincore/episode"}
        res = requests.delete(
            url,
            timeout=kwargs.pop("timeout", 30),
            params=params,
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if res.ok:
            return "ok"
        raise OcPyRequestException(
            "Could not delete metadata for event ("
            + self.get_identifier()
            + ")! ("
            + res.text
            + ")"
        )

    def get_publications(self, force_update=False, **kwargs) -> List[Publication]:
        if "publications" in self.data and not force_update:
            return [
                Publication(self.user, self.password, self.base_url + "/" + p["id"], p)
                for p in self.data["publications"]
            ]
        results = []
        url = self.base_url + "/publications"
        res = requests.get(
            url,
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if res.ok:
            for p in res.json():
                results.append(
                    Publication(
                        self.user,
                        self.password,
                        self.base_url + "/" + p["id"],
                        p,
                    )
                )
        return results

    def get_publication(self, publication_id, **kwargs) -> Optional[Publication]:
        url = self.base_url + "/publications/" + publication_id
        res = requests.get(
            url,
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if res.ok:
            p = res.json()
            return Publication(
                self.user, self.password, self.base_url + "/" + p["id"], p
            )
        return None

    def get_publication_by_channel(self, channel_name) -> Optional[Publication]:
        publications = self.get_publications()
        for p in publications:
            if p.get_channel() == channel_name:
                return p
        return None

    def get_api_channel_publication(self) -> Publication:
        return self.get_publication_by_channel("api")

    def get_engage_player_publication(self) -> Publication:
        return self.get_publication_by_channel("engage-player")

    def get_oai_pmh_default_publication(self) -> Publication:
        return self.get_publication_by_channel("oaipmh-default")

    def get_scheduling(self, force_update=False, **kwargs):
        if "scheduling" in self.data and not force_update:
            return self.data["scheduling"]
        url = self.base_url + "/scheduling"
        res = requests.get(
            url,
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if res.ok and res.status_code != 204:
            return Scheduling(**res.json())
        return None

    def set_scheduling(self, scheduling: Scheduling, allow_conflict=False, **kwargs):
        url = self.base_url + "/scheduling"
        # put request using application/x-ww-form-urlencoded
        form_data = {
            "scheduling": json.dumps(scheduling.get_scheduling_dict()),
            "allowConflict": allow_conflict,
        }

        res = requests.put(
            url,
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
            data=form_data,
            **kwargs,
        )
        if res.ok:
            self.data["scheduling"] = Scheduling(**scheduling.get_scheduling_dict())
            return None
        raise OcPyRequestException("could not set event scheduling!", response=res)

    def delete(self, **kwargs):
        url = self.base_url
        res = requests.delete(
            url,
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if res.ok:
            return "ok"
        raise OcPyRequestException(
            "Could not delete event (" + self.get_identifier() + ")! (" + res.text + ")"
        )


class EventsApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/api/events"

    def get_events(
        self,
        method="GET",
        limit=100,
        offset=0,
        sort="",
        sign=True,
        with_acl=False,
        with_metadata=False,
        with_scheduling=False,
        with_publications=False,
        events_filter=None,
        **kwargs,
    ) -> List[Event]:
        """
        :param method:
        :param limit:
        :param offset:
        :param sort:
        :param sign:
        :param with_acl:
        :param with_metadata:
        :param with_scheduling:
        :param with_publications:
        :param events_filter:
        :param kwargs:
        :return: List[Event]
        """
        parameters = {
            "limit": limit,
            "offset": offset,
            "sort": sort,
            "sign": sign,
            "withacl": with_acl,
            "withmetadata": with_metadata,
            "withscheduling": with_scheduling,
            "withpublications": with_publications,
        }
        if events_filter:
            if isinstance(events_filter, EventsApi.Filter):
                parameters["filter"] = events_filter.get_filter_string()
            else:
                parameters["filter"] = events_filter
        results = []
        res = requests.request(
            method,
            self.base_url,
            timeout=kwargs.pop("timeout", 60),
            auth=HTTPBasicAuth(self.user, self.password),
            params=parameters,
            **kwargs,
        )
        if res.ok:
            for event in res.json():
                results.append(
                    Event(
                        self.user,
                        self.password,
                        self.base_url + "/" + event["identifier"],
                        event,
                    )
                )
        else:
            if res.status_code == 500:
                logger.warning(
                    "Opencast responded with an internal server error (500)."
                )
            else:
                logger.error(
                    "Request to Opencast resulted in an error (code: {}): {}".format(
                        res.status_code, res.content
                    )
                )
            raise OcPyRequestException(res.content.decode("utf-8"), response=res)
        return results

    def get_all_events(
        self,
        sign=True,
        with_acl=False,
        with_metadata=False,
        with_scheduling=False,
        with_publications=False,
        sort=None,
        events_filter=None,
        batch_size=100,
        print_progress=False,
        initial_offset=0,
        generator=False,
    ) -> List[Event] | Generator[Event, None, None]:
        events_generator = self.get_all_events_generator(
            sign=sign,
            with_acl=with_acl,
            with_metadata=with_metadata,
            with_scheduling=with_scheduling,
            with_publications=with_publications,
            sort=sort,
            events_filter=events_filter,
            batch_size=batch_size,
            print_progress=print_progress,
            initial_offset=initial_offset,
        )
        if generator:
            return events_generator
        return list(events_generator)

    def get_all_events_generator(
        self,
        sign=True,
        with_acl=False,
        with_metadata=False,
        with_scheduling=False,
        with_publications=False,
        sort=None,
        events_filter=None,
        batch_size=100,
        print_progress=False,
        initial_offset=0,
    ) -> Generator[Event, None, None]:
        count = initial_offset
        while True:
            if print_progress:
                logger.info(f"requesting {batch_size} starting from offset {count}")
            res = self.get_events(
                sign=sign,
                with_acl=with_acl,
                sort=sort,
                with_metadata=with_metadata,
                with_scheduling=with_scheduling,
                with_publications=with_publications,
                events_filter=events_filter,
                limit=batch_size,
                offset=count,
            )
            if res is None or len(res) <= 0:
                logger.debug(
                    f"DONE; got 0 resuts; retrieved a total of {len(res)} events"
                )
                break
            logger.debug(f"got {len(res)} events")
            count += len(res)
            for r in res:
                yield r
        return

    def get_event(
        self,
        event_id,
        with_acl=False,
        with_metadata=False,
        with_scheduling=False,
        with_publications=False,
        **kwargs,
    ) -> Event:
        parameters = {
            "withacl": with_acl,
            "withmetadata": with_metadata,
            "withpublications": with_publications,
            "withscheduling": with_scheduling,
        }
        res = requests.get(
            self.base_url + "/" + event_id,
            timeout=kwargs.pop("timeout", 60),
            auth=HTTPBasicAuth(self.user, self.password),
            params=parameters,
            **kwargs,
        )
        if res.ok:
            event = res.json()
            return Event(
                self.user,
                self.password,
                self.base_url + "/" + event["identifier"],
                event,
            )
        logger.error(res.text)
        raise OcPyRequestException("could not get event!", response=res)

    def get_event_acl(self, event_id, **kwargs) -> List[ACL]:
        res = requests.get(
            self.base_url + f"/{event_id}/acl",
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
            **kwargs,
        )
        acls = []
        if res.ok:
            for a in res.json():
                acls.append(ACL(a["allow"], a["role"], Action(a["action"])))
            return acls
        raise OcPyRequestException("could not get event acls!", response=res)

    def get_event_metadata(self, event_id, **kwargs) -> dict:
        res = requests.get(
            self.base_url + f"/{event_id}/metadata",
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
            **kwargs,
        )
        if res.ok:
            return res.json()
        raise OcPyRequestException("could not get event metadata!", response=res)

    def get_event_publications(self, event_id, **kwargs) -> PublicationList:
        res = requests.get(
            self.base_url + f"/{event_id}/publications",
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
            **kwargs,
        )
        if res.ok:
            return PublicationList(res.json())
        raise OcPyRequestException("could not get event publications!", response=res)

    def get_event_scheduling(self, event_id, **kwargs) -> dict:
        res = requests.get(
            self.base_url + f"/{event_id}/scheduling",
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
            **kwargs,
        )
        if res.ok:
            return res.json()
        raise OcPyRequestException("could not get event scheduling!", response=res)

    def set_event_scheduling(
        self, event_id, scheduling: Scheduling, allow_conflict=False, **kwargs
    ) -> None:
        form_data = {
            "scheduling": json.dumps(scheduling.get_scheduling_dict()),
            "allowConflict": allow_conflict,
        }

        res = requests.put(
            self.base_url + f"/{event_id}/scheduling",
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
            data=form_data,
            **kwargs,
        )
        if res.ok:
            return None
        raise OcPyRequestException("could not set event scheduling!", response=res)

    def create_event(
        self, acl, metadata, processing, presenter_file, **kwargs
    ) -> Event:
        if isinstance(acl, List):
            acls = []
            for a in acl:
                if isinstance(a, ACL):
                    acls.append(a.get_acl_dict())
                else:
                    acls.append(a)
            acl = acls
        if isinstance(acl, ACL):
            acl = acl.get_acl_dict()
        data = {
            "acl": json.dumps(acl),
            "metadata": json.dumps(metadata),
            "processing": json.dumps(processing),
        }
        with open(presenter_file, "rb") as presenter_f:

            res = requests.post(
                self.base_url + "/",
                timeout=kwargs.pop("timeout", 30),
                auth=HTTPBasicAuth(self.user, self.password),
                data=data,
                files={"presenter": presenter_f},
                **kwargs,
            )
        if res.ok:
            event = res.json()
            return Event(
                self.user,
                self.password,
                self.base_url + "/" + event["identifier"],
                event,
            )
        logger.error(data)
        raise OcPyRequestException(
            "could not create event! (" + res.text + ")", response=res
        )

    def schedule_event(
        self, acl, metadata, scheduling, processing, **kwargs
    ) -> Union[Event, List[Event]]:
        """
        Creates a single event for a single scheduled recording. *It is not possible to schedule multiple recordings
        with this function / API endpoint, or is it?!*
        """
        if isinstance(acl, ACL):
            acl = [acl]
        if isinstance(acl, List):
            acls = []
            for a in acl:
                if isinstance(a, ACL):
                    acls.append(a.get_acl_dict())
                else:
                    acls.append(a)
            acl = acls

        data = {
            "acl": json.dumps(acl),
            "metadata": json.dumps(metadata),
            "processing": json.dumps(processing),
            "scheduling": json.dumps(scheduling),
        }
        logger.debug(f"sending {data} ")

        res = requests.post(
            self.base_url + "/",
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
            data=data,
            files=data,
            **kwargs,
        )
        if res.ok:
            event = res.json()
            if isinstance(event, list):
                evs = []
                for e in event:
                    evs.append(
                        Event(
                            self.user,
                            self.password,
                            self.base_url + "/" + e["identifier"],
                            e,
                        )
                    )
                return evs
            return Event(
                self.user,
                self.password,
                self.base_url + "/" + event["identifier"],
                event,
            )
        logger.error(f"status code: {res.status_code}", res.text)
        info = ""
        if res.status_code == 409:
            info = (
                "Additional info: (409) Conflict: (probably) conflicting events found)"
            )
        raise OcPyRequestException(
            f"could not create event! ({res.text}) - {info}",
            response=res,
            code=res.status_code,
        )

    def delete_event(self, event_id, **kwargs):
        res = requests.delete(
            self.base_url + "/" + event_id,
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
            **kwargs,
        )
        if res.ok:
            return "ok"
        raise OcPyRequestException("could not delete event! (" + res.text + ")")

    def get_events_part_of(self, series, limit=100, **kwargs) -> List[Event]:
        if not isinstance(series, str):
            series = series.get_identifier()
        return self.get_events(
            limit=limit,
            events_filter=EventsApi.Filter().set_series_filter(series),
            **kwargs,
        )

    def add_track(
        self,
        event_id: str,
        flavor: str,
        track_file,
        tags: List[str],
        overwrite_existing=False,
        **kwargs,
    ) -> str:
        """Adds a track to an event using a POST request and form data.
        Args:
            event_id (str): The ID of the event to which the track will be added.
            flavor (str): The flavor of the track (e.g., "captions/test").
            track_file (str): The path to the track file to be uploaded.
            tags (List[str]): A list of tags to be associated with the track.
            overwrite_existing (bool): Whether to overwrite an existing track with the same flavor. - Sadly this seems to not work
            yet, so you have to delete the track first if you want to overwrite it.
            **kwargs: Additional keyword arguments for the request.
        """

        url = f"{self.base_url}/{event_id}/track"
        files = {"track": (os.path.basename(track_file), open(track_file, "rb"))}
        data = {
            "eventId": event_id,
            "overwriteExisting": overwrite_existing,
            "flavor": flavor,
            "tags": ",".join(tags) if tags else "",
        }
        try:
            res = requests.post(
                url,
                timeout=kwargs.pop("timeout", 30),
                auth=HTTPBasicAuth(self.user, self.password),
                files=files,
                data=data,
                **kwargs,
            )
            if res.ok:
                return True
            raise OcPyRequestException(
                f"Could not add track to event {event_id}! ({res.text})",
                response=res,
            )
        except FileNotFoundError as e:
            raise OcPyRequestException(
                f"File not found: {track_file}. Please check the file path."
            ) from e
        except requests.RequestException as e:
            raise OcPyRequestException(
                f"An error occurred while adding the track: {str(e)}"
            ) from e

    class Filter:
        def __init__(self):
            self.filter_string = None

        def update_filter_string(self, update_string):
            if self.filter_string is None:
                self.filter_string = update_string
            else:
                self.filter_string += "," + update_string

        def set_contributors_filter(self, contributors):
            self.update_filter_string("contributors:{}".format(contributors))
            return self

        def set_location_filter(self, location):
            self.update_filter_string("location:{}".format(location))
            return self

        def set_series_filter(self, series):
            self.update_filter_string("series:{}".format(series))
            return self

        def set_subject_filter(self, subject):
            self.update_filter_string("subject:{}".format(subject))
            return self

        def set_start_filter(
            self,
            start_from: str | pendulum.DateTime,
            start_to: str | pendulum.DateTime,
        ):
            # example filter – start:2022-07-25T11:00:00.000Z/2022-07-25T14:00:00.000Z,subject:SS222182740
            if isinstance(start_from, pendulum.DateTime):
                start_from = start_from.isoformat()
            if isinstance(start_to, pendulum.DateTime):
                start_to = start_to.isoformat()
            self.update_filter_string(f"start:{start_from}/{start_to}")
            return self

        def set_text_filter_filter(self, text_filter):
            self.update_filter_string("textFilter:{}".format(text_filter))
            return self

        def get_filter_string(self):
            return self.filter_string



#  Copyright (c) 2019. Tobias Kurze

from typing import Union

import pytest

from ocpy.api.external import series
from ocpy.model.acl import ACL
import tests

_test_series: Union[None, series.Series] = None


@pytest.mark.usefixtures("setup_oc_connection")
class TestSeriesClass:
    def create_test_series(self):
        test_meta = [
            {
                "fields": [
                    {
                        "id": "title",
                        "label": "EVENTS.SERIES.DETAILS.METADATA.TITLE",
                        "type": "text",
                        "value": "Testserie_pytest",
                    }
                ],
                "flavor": "dublincore/series",
            }
        ]

        test_acl = ACL.get_read_write_acls("ROLE_TEST_USER")
        # test_acl = []

        return tests.SE_API.create_series(acl=test_acl, metadata=test_meta)

    def test_series_creation(self):
        print("testing series creation")
        seriess = tests.SE_API.get_all_series(generator=False)
        num_series = len(seriess)
        print(f"num_series before: {num_series}")
        new_series = self.create_test_series()
        new_s = tests.SE_API.get_all_series(generator=False)
        assert len(new_s) > num_series
        print(f"num_series after: {len(tests.SE_API.get_all_series(generator=False))}")
        new_series.delete()
        assert len(tests.SE_API.get_all_series(generator=False)) == num_series


"""
def setup_func():
    global test_event
    print("setup func ...")
    test_event = create_test_series()


def teardown_func():
    print("teardown func ...")
    global test_event
    print(test_event.delete())


@with_setup(setup_func, teardown_func)
def test_acl_add():
    return
    from pprint import pprint
    pprint(test_event.get_acl())
    num_acls = len(test_event.get_acl())
    print(num_acls)
    # new events can't be modified immediately (e.g. ACL), so wait...
    failed = False
    try_counter = 0
    while not failed:
        try:
            test_event.add_to_acl(ACL.get_read_acl("ROLE_ANONYMOUS"))
        except OcPyRequestException as e:
            if e.get_code() == 403:
                print("event may be processing ... can't modify ACL -> have to try again...(sleeping)")
                time.sleep(10)
                try_counter += 1
                if try_counter > 10:
                    failed = True
            else:
                failed = True
    assert len(test_event.get_acl()) > num_acls
"""

# @with_setup(setup_func, teardown_func)
# def tests():
#    ev_api.get_all_events()



# -*- coding: utf-8 -*-
"""Copyright (c) 2018. Tobias Kurze
This module demonstrates documentation as specified by the `Google Python
Style Guide`_. Docstrings may extend over multiple lines. Sections are created
with a section header and a colon followed by a block of indented text.

Example:
    Examples can be given using either the ``Example`` or ``Examples``
    sections. Sections support any reStructuredText formatting, including
    literal blocks::

        $ python example_google.py

Section breaks are created by resuming unindented text. Section breaks
are also implicitly created anytime a new section starts.

Attributes:
    module_level_variable1 (int): Module level variables may be documented in
        either the ``Attributes`` section of the module docstring, or in an
        inline docstring immediately following the variable.

        Either form is acceptable, but the two should not be mixed. Choose
        one convention to document module level variables and be consistent
        with it.

Todo:
    * For module TODOs
    * You have to also use ``sphinx.ext.todo`` extension

.. _Google Python Style Guide:
   http://google.github.io/styleguide/pyguide.html
"""
import os
from typing import Optional

import yaml
from loguru import logger
from requests import Response
from dotenv import load_dotenv

NAME = "ocpy"
CONNECTION_CONFIG = None


def get_connection_config():
    """Returns the connection parameters, that are being used by all get_*_api_client methods."""
    global CONNECTION_CONFIG  # pylint: disable=global-statement
    if CONNECTION_CONFIG is None:
        logger.warning(
            "connection_config is not set – please call >setup_connection_config< before using the API client(s)"
        )
        logger.info("trying to setup connection_config from environment...")
        CONNECTION_CONFIG = setup_connection_config_from_env()
    return CONNECTION_CONFIG


def setup_connection_config(
    user: str = "admin",
    password: str = None,
    server_url: str = "http://opencast.ubka.uni-karlsruhe.de:8080",
    digest_user: str = "opencast_system_account",
    digest_password: str = None,
):
    """Sets up the connection parameters, that are being used by all get_*_api_client methods.

    Args:
        user (str): username to use for the connection
        password (str): password to use for the connection
        server_url (str): base server_url (incl. port if non standard) to use for the connection
        digest_user (str): username for digest authentication – needed for certain APIs / tasks
        digest_password (str): password of user for digest authentication

    Returns:
        dict: the connection_config dictionary
    """
    global CONNECTION_CONFIG  # pylint: disable=global-statement
    if server_url[-1] == "/":
        server_url = server_url[0:-1]
    CONNECTION_CONFIG = {
        "user": user,
        "password": password,
        "server_url": server_url,
        "digest_user": digest_user,
        "digest_password": digest_password,
    }
    return CONNECTION_CONFIG


def setup_connection_config_from_yaml_file(config_yaml: str, encoding="utf-8") -> dict:
    """Setup the connection parameters from a provided config file in yaml format.

    `PEP 484`_ type annotations are supported. If attribute, parameter, and
    return types are annotated according to `PEP 484`_, they do not need to be
    included in the docstring:

    Args:
        config_yaml (str): Config file path

    Returns:
        dict: the connection_config dictionary

    .. _PEP 484:
        https://www.python.org/dev/peps/pep-0484/

    """
    try:
        with open(config_yaml, "r", encoding=encoding) as stream:
            try:
                return setup_connection_config(**yaml.safe_load(stream))
            except yaml.YAMLError:
                logger.exception("Invalid yaml file")
            except TypeError:
                logger.exception(
                    "Invalid settings in YAML file – user, password, server_url must be specified"
                )
    except FileNotFoundError:
        logger.exception(f"Specified file {config_yaml} not found!")


def setup_connection_config_from_env(read_dotenv: bool = True) -> dict:
    """Setup the connection parameters from environment variables."""
    if read_dotenv:
        load_dotenv()

    oc_user = os.getenv("OC_USER", "admin")
    oc_password = os.getenv("OC_PASSWORD", os.getenv("OC_PW", None))
    oc_url = os.getenv("OC_URL", os.getenv("OC_SERVER", None))
    oc_digest_user = os.getenv("OC_DIGEST_USER", "opencast_system_account")
    oc_digest_user_password = os.getenv("OC_DIGEST_USER_PASSWORD", None)

    if oc_password is None or oc_url is None:
        raise ValueError("OC_PASSWORD or OC_URL is not set")
    if oc_digest_user_password is None:
        logger.warning("OC_DIGEST_USER_PASSWORD is not set")

    return setup_connection_config(
        user=oc_user,
        password=oc_password,
        server_url=oc_url,
        digest_user=oc_digest_user,
        digest_password=oc_digest_user_password,
    )


class OcPyException(Exception):
    """Base class for exceptions in this module."""


class OcPyRequestException(OcPyException):
    """Exception raised for errors in this module linked to requests."""

    def __init__(
        self, message, code: Optional[int] = None, response: Optional[Response] = None
    ):
        super().__init__(message)
        self.code = code
        self.response = response
        if response is not None:
            if self.code is None:
                self.code = response.status_code

    def get_code(self) -> Optional[int]:
        """Return the exception code."""
        return self.code

    def is_opencast_error(self) -> bool:
        """Return True if the exception is an error returned by Opencast (http: 500)."""
        return self.code == 500

    def get_response(self) -> Optional[Response]:
        """Return the response object."""
        return self.response



from ocpy import get_connection_config


class OpenCastBaseApiClient:
    def __init__(self, user: str = None, password: str = None, server_url: str = None):
        if user is None or password is None or server_url is None:
            connection_config = get_connection_config()
            if connection_config is None:
                raise ValueError(
                    "user, password and server_url must be specified; directly or by calling "
                    "ocpy.api.setup_connection_config(user, password,server_url)!"
                )
            user = connection_config.get("user", user) if user is None else user
            password = connection_config.get("password", password) if password is None else password
            server_url = connection_config.get("server_url", server_url) if server_url is None else server_url
            if user is None or password is None or server_url is None:
                raise ValueError(
                    "invalid values in connection_config, please call "
                    "ocpy.api.setup_connection_config(user, password,server_url) again"
                )
        self.user = user
        self.password = password
        self.server_url = server_url


class OpenCastDigestBaseApiClient:
    def __init__(
            self,
            digest_user: str = None,
            digest_password: str = None,
            server_url: str = None,
            optional: bool = False
    ):
        if digest_user is None or digest_password is None or server_url is None:
            connection_config = get_connection_config()
            if connection_config is None and not optional:
                raise ValueError(
                    "digest_user, digest_password and server_url must be specified; directly or by calling "
                    "ocpy.api.setup_connection_config(digest_user, digest_password, server_url)!"
                )
            digest_user = connection_config.get("digest_user", digest_user) if digest_user is None else digest_user
            digest_password = connection_config.get("digest_password", digest_password) if digest_password is None else digest_password
            server_url = connection_config.get("server_url", server_url) if server_url is None else server_url
            if (digest_user is None or digest_password is None or server_url is None) and not optional:
                raise ValueError(
                    "invalid values in connection_config, please call "
                    "ocpy.api.setup_connection_config(digest_user, digest_password, server_url, etc.) again"
                )
        self.server_url = server_url
        self.digest_user = digest_user
        self.digest_password = digest_password


class OpenCastApiService:
    def __init__(self, service_url):
        from ocpy.api.service import Service
        if isinstance(service_url, Service):
            service_url = service_url.get_url()
        self.base_url = service_url
        self.server_url = service_url

    def __str__(self):
        return self.base_url

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
from typing import Optional

from loguru import logger
from requests import Session
from requests.auth import HTTPDigestAuth, HTTPBasicAuth
from pprint import pprint

from ocpy.api.api_client import OpenCastDigestBaseApiClient, OpenCastBaseApiClient, OpenCastApiService


class WorkflowApi(OpenCastApiService, OpenCastBaseApiClient,
                  OpenCastDigestBaseApiClient):
    def __init__(self,
                 service_url,
                 user=None,
                 password=None,
                 digest_user=None,
                 digest_password=None,
                 use_digest_auth=False,
                 **_kwargs):
        OpenCastApiService.__init__(self, service_url)
        OpenCastBaseApiClient.__init__(self, user, password)
        OpenCastDigestBaseApiClient.__init__(self, digest_user,
                                             digest_password, optional=True)

        self.use_digest_auth = self.use_digest_auth

        self.session = Session()
        if self.use_digest_auth:
            self.session.auth = HTTPDigestAuth(self.digest_user,
                                               self.digest_password)
            self.session.headers.update({"X-Requested-Auth": "Digest"})
        else:
            self.session.auth = HTTPBasicAuth(self.user, self.password)

    def get_count(self, **kwargs) -> int:
        url = self.base_url + "/count"
        res = self.session.get(
            url,
            **kwargs
        )
        if res.ok:
            return int(res.text)

        raise Exception("Could not get workflow count!")

    def get_definitions(self, as_json=True):
        if as_json:
            url = self.base_url + "/definitions.json"
        else:
            url = self.base_url + "/definitions.xml"
        res = self.session.get(url)
        if res.ok:
            if as_json:
                return res.json()
            else:
                return res.content.decode("utf-8")
        raise Exception("Could not get workflow definitions!")

    def get_handlers(self):
        url = self.base_url + "/handlers.json"
        res = self.session.get(url)
        if res.ok:
            return res.json()
        raise Exception("Could not get workflow definitions!")

    def get_statistics(self, as_json=True):
        if as_json:
            url = self.base_url + "/statistics.json"
        else:
            url = self.base_url + "/statistics.xml"
        res = self.session.get(url)
        if res.ok:
            if as_json:
                return res.json()
            else:
                return res.content.decode("utf-8")
        raise Exception("Could not get workflow definitions!")

    def get_instances(self, as_json=True):
        if as_json:
            url = self.base_url + "/instances.json"
        else:
            url = self.base_url + "/instances.xml"
        res = self.session.get(url)
        if res.ok:
            if as_json:
                return res.json()
            else:
                return res.content.decode("utf-8")
        raise Exception("Could not get workflow definitions!")

    def get_instance(self, id: str, as_json=True):
        if as_json:
            url = self.base_url + "/instances.json"
        else:
            url = self.base_url + "/instances.xml"
        res = self.session.get(url)
        if res.ok:
            if as_json:
                return res.json()
            else:
                return res.content.decode("utf-8")
        raise Exception("Could not get workflow definitions!")


def main():
    from ocpy.api.service import ServiceApi, ServiceType

    s_api = ServiceApi(server_url="http://opencast-dev.bibliothek.kit.edu:8080")
    s_api = ServiceApi(
        server_url="http://localhost:8080", user="admin", password="opencast"
    )
    # s_api = ServiceApi()
    wf_service = s_api.get_available(ServiceType.workflow)[0]

    print(wf_service)
    print(type(wf_service))
    api = WorkflowApi(wf_service)
    print(api)
    print(api.get_count())
    # print(api.get_definitions(as_json=False))
    # pprint(api.get_definitions())
    # pprint(api.get_handlers())
    pprint(api.get_instances())


if __name__ == "__main__":
    main()



import requests
from requests.auth import HTTPBasicAuth
import json
from pprint import pprint

from typing import List

from ocpy.api.api_client import OpenCastBaseApiClient


class Agent:
    def __init__(self, user, password, url, data):
        self.base_url = url
        self.user = user
        self.password = password
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_agent_id(self) -> str:
        return self.data["agent_id"]

    def get_status(self) -> str:
        return self.data["status"]

    def get_inputs(self) -> List[str]:
        return self.data["inputs"]

    def get_update(self) -> str:
        return self.data["update"]

    def get_url(self) -> str:
        return self.data["url"]


class AgentsApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **_kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/api/agents/"

    def get_agents(self, limit=100, offset=0, **kwargs) -> List[Agent]:
        """
        :param limit:
        :param offset:
        :param kwargs:
        :return: List[Series]
        """
        parameters = {"limit": limit, "offset": offset}
        results = []
        res = requests.get(
            self.base_url,
            auth=HTTPBasicAuth(self.user, self.password),
            params=parameters,
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            for a in res.json():
                results.append(
                    Agent(self.user, self.password, self.base_url + a["agent_id"], a)
                )
        return results

    def get_all_agents(self) -> List[Agent]:
        result = []
        while True:
            res = self.get_agents(limit=100, offset=len(result))
            if res is None or len(res) <= 0:
                break
            result.extend(res)
        return result



#  Copyright (c) 2019. Tobias Kurze

import json


class Server:
    def __init__(self, name: str, port: str):
        self.name = name
        self.port = port

    def __str__(self):
        return json.dumps({"name": self.name, "port": self.port})

    def __repr__(self):
        return self.__str__()

    def get_name(self):
        return self.name

    def get_port(self):
        return self.port



# -*- coding: utf-8 -*-
"""Opencast admin_ui IndexApiAPI.

Base implementation of opencast admin_ui index API.
"""
import requests
from requests.auth import HTTPBasicAuth
from ocpy import OcPyException

from ocpy.api.api_client import OpenCastBaseApiClient


class IndexApi(OpenCastBaseApiClient):
    """API to manage the admin ui index.

    Basically for clearing and recreating the index.
    """

    def __init__(self, user: str = None, password: str = None, server_url: str = None):
        """Create new index API instance.

        Args:
            user (str): Opencast username
            password (str): Opencast password
            server_url (str): Base URL of Opencast Admin Node
        """
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/index"

    def clear_index(self, timeout=10, **kwargs):
        """Clear the index for the admin UI.

        This is useful if the UI returns
        events that don't exist anymore. There is another clearIndex function
        for the external API. Also the engage player uses another endpoint
        (the search endpoint) which can't be cleared, but single items can be
        deleted from search using the search
        endpoint's delete function.

        Returns:
            str: "ok" if everything went well
        Raises:
            Exception: if index could not be cleared; also in case of
            connection errors
        """
        url = self.base_url + "/clear"
        res = requests.post(
            url, timeout=timeout, auth=HTTPBasicAuth(self.user, self.password), **kwargs
        )
        if res.ok:
            return "ok"
        raise OcPyException(f"Could not get clear index! {res.text}")

    def recreate_index(self, service=None, timeout=10, **kwargs) -> str:
        """Recreate the index for the admin UI (or for a specific service).

        Allowed services are: Groups, Acl, Themes, Series, Scheduler,
        workflow, AssetManager and Comments. The service order (see above) is
        very important! Make sure, you do not run index rebuild for more than
        one service at a time! The service parameter can be omitted.

        This is useful if the UI returns events that don't exist anymore.
        There is another recreateIndex function for the external API. Also the
        engage player uses another endpoint (the search endpoint) which can't
        be recreated, but single items can be deleted from search using the
        search endpoint's delete function.

        Args:
            service (object):
        Returns:
            str: "ok" if everything went well
        Raises:
            Exception: if index could not be recreated; also in case of
            connection errors
        """
        url = self.base_url + "/rebuild"
        if service is not None:
            url += "/" + service
        res = requests.post(
            url, timeout=timeout, auth=HTTPBasicAuth(self.user, self.password), **kwargs
        )
        if res.ok:
            return "ok"
        raise OcPyException(f"Could not rebuild index! {res.text}")



import requests
from pprint import pprint
from requests.auth import HTTPBasicAuth
from loguru import logger

from ocpy import OcPyRequestException
from ocpy.model.mediapackage import MediaPackage
from ocpy.api.api_client import OpenCastBaseApiClient


class SearchApi(OpenCastBaseApiClient):
    """Search API class.
    PLEASE NOTE: The search API is (usually) only available on the presentation node.
    (Which might or might not be the same as the admin node)!
    Args:
        OpenCastBaseApiClient (_type_): _description_
    """

    def __init__(self, user=None, password=None, server_url=None, **_kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/search"
        if self.base_url and "admin" in self.base_url.lower():
            logger.warning(
                "Search API is usually only available on the presentation node! Requsets might fail!"
            )

    def get_episode(
        self,
        e_id: str = "",
        q: str = "",
        series_id: str = "",
        series_name: str = "",
        sort: str = "",
        limit: int = 20,
        offset: int = 0,
        admin: bool = False,
        sign: bool = True,
        resp_format: str = "json",
        **_kwargs,
    ):
        params = {
            "id": e_id,
            "q": q,
            "sid": series_id,
            "sname": series_name,
            "sort": sort,
            "limit": limit,
            "offset": offset,
            "admin": admin,
            "sign": sign,
        }
        res = requests.get(
            self.base_url + f"/episode.{resp_format}",
            params=params,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=10
        )
        if res.ok:
            if resp_format == "json":
                return res.json().get("search-results", {}).get("result", [])
            return res.text
        raise OcPyRequestException("could not get episode(s)!", response=res)

    def get_lucene(
        self,
        query: str = "",
        series: bool = False,
        sort: str = "",
        limit: int = 20,
        offset: int = 0,
        admin: bool = False,
        sign: bool = True,
        resp_format="json",
        **kwargs,
    ):
        params = {
            "q": query,
            "series": series,
            "sort": sort,
            "limit": limit,
            "offset": offset,
            "admin": admin,
            "sign": sign,
        }

        res = requests.get(
            self.base_url + f"/lucene.{resp_format}",
            auth=HTTPBasicAuth(self.user, self.password),
            params=params,
            timeout=10,
            **kwargs,
        )

        if res.ok:
            if resp_format == "json":
                return res.json()
            return res.text
        raise OcPyRequestException("could not get response!", response=res)

    def get_series(
        self,
        s_id: str = "",
        query: str = "",
        episodes: bool = False,
        sort: str = "",
        limit: int = 20,
        offset: int = 0,
        admin: bool = False,
        sign: bool = True,
        resp_format: str = "json",
        **kwargs,
    ):
        params = {
            "id": s_id,
            "q": query,
            "episodes": episodes,
            "sort": sort,
            "limit": limit,
            "offset": offset,
            "admin": admin,
            "sign": sign,
        }

        res = requests.get(
            self.base_url + f"/lucene.{resp_format}",
            auth=HTTPBasicAuth(self.user, self.password),
            params=params,
            timeout=10,
            **kwargs,
        )

        if res.ok:
            if resp_format == "json":
                return res.json()
            return res.text
        raise OcPyRequestException("could not get series!", response=res)

    def add_mediapackage(self, media_package_xml: str):
        url = self.base_url + "/add"
        data = {"mediapackage": media_package_xml}
       
        res = requests.post(
            url,
            data=data,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=10,
        )
        if res.ok:
            return MediaPackage(res.text)
        raise OcPyRequestException("could not get event!", response=res)

    def delete_mediapackage(self, mp_id: str):
        url = self.base_url + f"/{mp_id}"
        res = requests.delete(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=10,
        )
        if res.ok:
            return res.text
        raise OcPyRequestException("could not delete mediapackage!", response=res)


def main():
    api = SearchApi()
    pprint(api.get_episode())
    pprint(api.get_lucene("mh_default_org"))


if __name__ == "__main__":
    main()



#  Copyright (c) 2019. Tobias Kurze

import json
import re
from collections import OrderedDict
from pprint import pprint
from typing import List, Optional

d = {
    "@id": "68e9fcb3-0662-43dd-a957-ba9bc1414195",
    "@type": "presentation/source",
    "audio": {
        "@id": "audio-1",
        "bitrate": "83050.0",
        "channels": "1",
        "device": None,
        "encoder": {
            "@type": "AAC (Advanced Audio Coding)"
        },
        "framecount": "261",
        "samplingrate": "48000"
    },
    "checksum": {
        "#text": "a3ac7ddabb263c2d00b73e8177d15c8d",
        "@type": "md5"
    },

    "duration": "5568",
    "live": "false",
    "mimetype": "video/mp4",
    "size": "383631",
    "tags": {
        "tag": "archive"
    },
    "url": "https://oc-bib-admin.bibliothek.kit.edu/assets/assets/983d0f7c-f81c-473a-80c2-e5a9b497ae99/68e9fcb3-0662-43dd-a957-ba9bc1414195/3/vid1_hqt.mp4",
    "video": {
        "@id": "video-1",
        "bitrate": "465641.0",
        "device": None,
        "encoder": {
            "@type": "H.264 / AVC / MPEG-4 AVC / MPEG-4 part 10"
        },
        "framecount": "166",
        "framerate": "30.0",
        "resolution": "560x320"
    }
}


class Publication:
    def __init__(self, data):
        self.data = data
        self._fix_dict_keys()

    def __str__(self):
        return json.dumps(self.data)

    def __repr__(self):
        return self.__str__()

    def _fix_dict_keys(self, d: Optional[dict] = None):
        if d is None:
            d = self.data
        problematic_keys: List[str] = [k for k in d.keys() if
                                       any([s in k for s in ['#', '@']])]
        for p_k in problematic_keys:
            nice_key = p_k.replace('#', '').replace('@', '')
            d[nice_key] = d.pop(p_k)
        for k in d.keys():  # recurse
            if isinstance(d[k], dict) or isinstance(d[k], OrderedDict):
                self._fix_dict_keys(d[k])
            if isinstance(d[k], List) and len(d[k]) > 0 and isinstance(d[k][0], dict):
                [self._fix_dict_keys(e) for e in d[k]]

    def get_identifier(self):
        return self.data.get("identifier")

    def get_channel(self):
        return self.data.get("channel")

    def get_media(self):
        return self.data.get("media")

    def get_attachments(self):
        return self.data.get("attachments")

    def get_tags(self):
        return self.data.get("tags")

    def get_metadata(self):
        return self.data.get("metadata")

    def get_url(self):
        return self.data.get("url")


class PublicationList:
    def __init__(self, data: List[dict]):
        self.publications = []
        for p in data:
            self.publications.append(Publication(p))

    def get_publications(self) -> List[Publication]:
        return self.publications
    
    def get_publication_channels(self) -> List[str]:
        return [p.get_channel() for p in self.publications]

    def get_publication_by_channel(self, channel: str = "internal") -> \
    Optional[Publication]:
        for p in self.publications:
            if channel == p.get_channel():
                return p
        return None


if __name__ == '__main__':
    pprint(d)
    pprint(Publication(d).data)



import logging
import unittest

import pytest

from ocpy.api.admin_ui.service import AdminServiceApi


@pytest.mark.usefixtures("setup_oc_connection")
class TestServiceClass(unittest.TestCase):
    def test_get_services(self):
        service_api = AdminServiceApi()
        self.assertIsNotNone(service_api.get_services())
        self.assertIsInstance(service_api.get_services(), dict)



#  Copyright (c) 2019. Tobias Kurze

import json

from ocpy.model.server import Server


class Organization:
    def __init__(self, id, name, servers, admin_role, anonymous_role, properties):
        self.id = id
        self.name = name
        self.servers = servers
        self.admin_role = admin_role
        self.anonymous_role = anonymous_role
        self.properties = properties

    def __str__(self):
        return json.dumps(
            {
                "id": self.id,
                "name": self.name,
                "servers": self.servers,
                "adminRole": self.admin_role,
                "anonymousRole": self.anonymous_role,
                "properties": self.properties,
            }
        )

    def __repr__(self):
        return self.__str__()

    def get_id(self) -> str:
        return self.id

    def get_name(self) -> str:
        return self.name

    def get_servers(self):
        print(self.servers)
        servers = []
        if isinstance(self.servers, dict):
            servers.append(Server(**self.servers["server"]))
        else:
            for s in self.servers:
                servers.append(Server(**s["server"]))
        return servers

    def get_admin_role(self) -> str:
        return self.admin_role

    def get_anonymous_role(self) -> str:
        return self.anonymous_role

    def get_properties(self):
        return self.properties



import requests
from requests.auth import HTTPBasicAuth
import json

from ocpy.api.api_client import OpenCastBaseApiClient
import os


class User:
    def __init__(self, user, password, url, data):
        self.base_url = url
        self.user = user
        self.password = password
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_email(self) -> str:
        return self.data["email"]

    def get_name(self) -> str:
        return self.data["name"]

    def get_provider(self) -> str:
        return self.data["provider"]

    def get_username(self) -> str:
        return self.data["username"]

    def get_user_role(self) -> str:
        return self.data["userRole"]


class UploadApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/upload"

    def get_job(self, job_id, **kwargs) -> dict:
        res = requests.get(
            self.base_url + "/" + job_id + ".json",
            auth=HTTPBasicAuth(self.user, self.password),
            **kwargs
        )
        if res.ok:
            return res.json()
        raise Exception("Could not get job info!")

    def create_new_job(
        self,
        file_name=None,
        file_size=None,
        chunk_size=None,
        flavor=None,
        mediapackage=None,
        **kwargs
    ):

        data = {
            "filename": file_name,
            "filesize": file_size,
            "chunksize": chunk_size,
            "flavor": flavor,
            "mediapackage": mediapackage,
        }

        res = requests.post(
            self.base_url + "/newjob",
            auth=HTTPBasicAuth(self.user, self.password),
            files=data,
            **kwargs
        )
        if res.ok:
            return res.text
        print(res.content)
        raise Exception("Could not create new upload!")

    def append_chunk_to_job(self, job_id, file_data, chunk_number=None, **kwargs):
        data = {"chunknumber": chunk_number, "filedata": file_data}

        res = requests.post(
            self.base_url + "/job/" + job_id,
            auth=HTTPBasicAuth(self.user, self.password),
            files=data,
            **kwargs
        )

        if res.ok:
            return res.text
        print(res.content)
        raise Exception("Could not append chunk to job!")


def read_in_chunks(file_object, chunk_size=1024):
    """Lazy function (generator) to read a file piece by piece.
    Default chunk size: 1k."""
    while True:
        data = file_object.read(chunk_size)
        if not data:
            break
        yield data


def test_file_upload():
    api = UploadApi()
    file = "/home/tobias/Downloads/lot_small_files.zip"
    print(os.path.getsize(file))
    job_id = api.create_new_job(
        file_name=os.path.basename(file), file_size=os.path.getsize(file)
    )
    f = open(file, "rb")
    count = 0
    for piece in read_in_chunks(f):
        print(count)
        api.append_chunk_to_job(job_id, piece)
        count += 1


def main():
    test_file_upload()
    return
    api = UploadApi()
    job_id = api.create_new_job()
    print(api.append_chunk_to_job(job_id, "file_data"))


if __name__ == "__main__":
    main()



"""Test for the index API"""

import unittest
import pytest

from ocpy.api.external.index import IndexApi


@pytest.mark.usefixtures("setup_oc_connection")
# @pytest.mark.skip(reason="deprecated API")
class TestIndexClass(unittest.TestCase):
    """Test the index API."""

    def test_get_services(self):
        """Test recreating index services."""
        ix_api = IndexApi()
        out = ix_api.recreate_index()
        self.assertEqual(out, "ok")



#  Copyright (c) 2023. Tobias Kurze
"""Search API tests."""
import pytest
import tests

from loguru import logger

TEST_URL = "https://oc-alpha-delivery-1.bibliothek.kit.edu/staticfiles/mh_default_org/engage-player/ea248461-b144-4349-9c12-da22633c14d4/8e25b7d9-b501-4e21-a74e-b956d31fccdc/CS_10_50_Bauingenieu_gro_er_hs_test_reihe_20230119_160500_S1R1.mp4"


@pytest.mark.usefixtures("setup_oc_connection")
class TestSignApiClass:
    """Test the search API."""

    def test_sign_api_accepts_url(self):
        """Get episodes from search API."""
        accepts = tests.SIGN_API.accepts(TEST_URL)
        logger.info(accepts)
        assert isinstance(accepts, bool)

    def test_sign_api_sign_url(self):
        """Get episodes from search API."""
        url = tests.SIGN_API.sign(TEST_URL)
        logger.info(url)
        assert isinstance(url, str)



"""OpenCast Groups API Client Module"""

import json
from typing import List

import requests
from requests.auth import HTTPBasicAuth

from ocpy.api.api_client import OpenCastBaseApiClient


class Group:
    """Group class representing a group in OpenCast."""

    def __init__(self, user, password, url, data):
        self.base_url = url
        self.user = user
        self.password = password
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_identifier(self) -> str:
        return self.data["identifier"]

    def get_role(self) -> str:
        return self.data["role"]

    def get_organization(self) -> List[str]:
        return self.data["organization"]

    def get_roles(self) -> str:
        return self.data["roles"]

    def get_members(self) -> str:
        return self.data["members"]

    def get_name(self) -> str:
        return self.data["name"]

    def get_description(self) -> str:
        return self.data["description"]

    def add_member(self, username: str, **kwargs):
        res = requests.post(
            self.base_url + "/members",
            auth=HTTPBasicAuth(self.user, self.password),
            data={"member": username},
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        if res.ok:
            return res.content
        return False

    def remove_member(self, username: str, **kwargs):
        res = requests.delete(
            self.base_url + "/members/" + username,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        return res.ok

    def update_group(
        self,
        name: str = None,
        description: str = None,
        roles: str = None,
        members: str = None,
        **kwargs,
    ):
        if name is None:
            name = self.data["name"]
        res = requests.put(
            self.base_url,
            auth=HTTPBasicAuth(self.user, self.password),
            data={
                "name": name,
                "description": description,
                "roles": roles,
                "members": members,
            },
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        return res.ok

    def delete(self, **kwargs):
        res = requests.delete(
            self.base_url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        return res.ok


class GroupsApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **_kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/api/groups/"

    def get_groups(self, limit=100, offset=0, **kwargs) -> List[Group]:
        """
        :param limit:
        :param offset:
        :param kwargs:
        :return: List[Series]
        """
        parameters = {"limit": limit, "offset": offset}
        results = []
        res = requests.get(
            self.base_url,
            auth=HTTPBasicAuth(self.user, self.password),
            params=parameters,
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        if res.ok:
            for g in res.json():
                results.append(
                    Group(self.user, self.password, self.base_url + g["identifier"], g)
                )
        return results

    def get_all_groups(self) -> List[Group]:
        result = []
        while True:
            res = self.get_groups(limit=100, offset=len(result))
            if res is None or len(res) <= 0:
                break
            result.extend(res)
        return result

    def create_group(
        self,
        name: str,
        description: str = None,
        roles: str = None,
        members: str = None,
        **kwargs,
    ):
        res = requests.post(
            self.base_url,
            auth=HTTPBasicAuth(self.user, self.password),
            data={
                "name": name,
                "description": description,
                "roles": roles,
                "members": members,
            },
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        return res.ok

    def get_group(self, identifier, **kwargs):
        res = requests.get(
            self.base_url + identifier,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        g = res.json()
        return Group(self.user, self.password, self.base_url + g["identifier"], g)



import os
import unittest
import logging

from ocpy import setup_connection_config_from_yaml_file


class TestApiModuleClass(unittest.TestCase):
    def test_setup_connection_config_from_yaml_file(self):
        config = setup_connection_config_from_yaml_file(
            os.path.abspath("tests/assets/example_connection_conf.yml")
        )
        logging.debug(config)
        self.assertIsNotNone(config)
        self.assertIsInstance(config, dict)
        self.assertIn("server_url", config)



import os

import requests
from loguru import logger
from requests.auth import HTTPBasicAuth
from typing import List, Optional, Union
import json
from pprint import pprint

from ocpy.api.api_client import OpenCastBaseApiClient
from ocpy import OcPyRequestException
from ocpy.model.event_models import Publication
from ocpy.model.acl import ACL, Action


class WorkflowInstance:
    def __init__(self, user, password, url, data):
        self.base_url = url
        self.user = user
        self.password = password
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4,
                          separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_creator(self):
        return self.data["creator"]

    def get_description(self):
        return self.data["description"]

    def get_event_identifier(self):
        return self.data["event_identifier"]

    def get_identifier(self):
        return self.data["identifier"]

    def get_state(self):
        return self.data["state"]

    def get_title(self):
        return self.data["title"]

    def get_workflow_definition_identifier(self):
        return self.data["workflow_definition_identifier"]

    def get_configuration(self) -> Union[dict, None]:
        return self.data.get("configuration", None)

    def get_operations(self) -> Union[dict, None]:
        return self.data.get("operations", None)

    def delete(self, **kwargs):
        url = self.base_url
        res = requests.delete(url,
                              auth=HTTPBasicAuth(self.user, self.password),
                              timeout=kwargs.pop("timeout", 30),)
        if res.ok:
            return "ok"
        raise Exception(
            "Could not delete event (" + self.get_identifier() + ")! (" + res.text + ")"
        )


class WorkflowsApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/api/workflows"

    def get_workflow_instances(
            self,
            method="GET",
            limit=100,
            offset=0,
            sign=True,
            with_acl=False,
            with_metadata=False,
            with_publications=False,
            events_filter=None,
            **kwargs
    ) -> List[WorkflowInstance]:
        """
        :param method:
        :param limit:
        :param offset:
        :param sign:
        :param with_acl:
        :param with_metadata:
        :param with_publications:
        :param events_filter:
        :param kwargs:
        :return: List[Event]
        """
        parameters = {
            "limit": limit,
            "offset": offset,
            "sign": sign,
            "withacl": with_acl,
            "withmetadata": with_metadata,
            "withpublications": with_publications,
        }
        if events_filter:
            if isinstance(events_filter, WorkflowsApi.Filter):
                parameters["filter"] = events_filter.get_filter_string()
            else:
                parameters["filter"] = events_filter
        results = []
        res = requests.request(
            method,
            self.base_url,
            auth=HTTPBasicAuth(self.user, self.password),
            params=parameters,
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            for wfi in res.json():
                pprint(wfi)
                results.append(
                    WorkflowInstance(
                        self.user,
                        self.password,
                        self.base_url + "/" + str(wfi["identifier"]),
                        wfi,
                    )
                )
        else:
            if res.status_code == 500:
                logger.warning(
                    "Opencast responded with an internal server error (500)."
                )
            else:
                logger.error(
                    "Request to Opencast resulted in an error (code: {}): {}".format(
                        res.status_code, res.content
                    )
                )
            raise OcPyRequestException(res.content.decode("utf-8"),
                                       response=res)
        return results

    def get_workflow_instance(
            self, wfi_id: int, with_operations=False, with_configuration=False,
            **kwargs
    ) -> WorkflowInstance:
        parameters = {
            "withoperations": with_operations,
            "withconfiguration": with_configuration,
        }
        res = requests.get(
            self.base_url + "/" + str(wfi_id),
            auth=HTTPBasicAuth(self.user, self.password),
            params=parameters,
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            wfi = res.json()
            return WorkflowInstance(
                self.user,
                self.password,
                self.base_url + "/" + str(wfi["identifier"]),
                wfi,
            )
        raise OcPyRequestException("could not get wfi!", response=res)

    def create_workflow_instance(
            self,
            event_identifier,
            workflow_definition_identifier,
            configuration=None,
            with_operations=False,
            with_configuration=False,
            **kwargs
    ) -> WorkflowInstance:

        if configuration is not None:
            if not isinstance(configuration, str):
                configuration = json.dumps(configuration)

        data = {
            "event_identifier": event_identifier,
            "workflow_definition_identifier": workflow_definition_identifier,
            "configuration": configuration,
            "with_operations": with_operations,
            "with_configuration": with_configuration,
        }

        res = requests.post(
            self.base_url + "/",
            auth=HTTPBasicAuth(self.user, self.password),
            data=data,
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            wfi = res.json()
            return WorkflowInstance(
                self.user,
                self.password,
                self.base_url + "/" + str(wfi["identifier"]),
                wfi,
            )
        raise OcPyRequestException(
            "could not create workflow instance! (" + res.text + ")",
            response=res
        )

    def delete_workflow_instance(self, wfi_id, **kwargs):
        res = requests.delete(
            self.base_url + "/" + str(wfi_id),
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return "ok"
        raise OcPyRequestException(
            "could not delete event! (" + res.text + ")")

    class Filter:
        def __init__(self):
            self.filter_string = None

        def update_filter_string(self, update_string):
            if self.filter_string is None:
                self.filter_string = update_string
            else:
                self.filter_string += "," + update_string

        def set_contributors_filter(self, contributors):
            self.update_filter_string("contributors:{}".format(contributors))
            return self

        def set_location_filter(self, location):
            self.update_filter_string("location:{}".format(location))
            return self

        def set_series_filter(self, series):
            self.update_filter_string("series:{}".format(series))
            return self

        def set_subject_filter(self, subject):
            self.update_filter_string("subject:{}".format(subject))
            return self

        def set_text_filter_filter(self, text_filter):
            self.update_filter_string("textFilter:{}".format(text_filter))
            return self

        def get_filter_string(self):
            return self.filter_string



#  Copyright (c) 2019. Tobias Kurze

from ocpy.model.acl import ACL, Action


def test_read_acl():
    acl = ACL(True, "ROLE_TEST", Action.get_read_action())
    assert str(acl) == '{"allow": true, "role": "ROLE_TEST", "action": "read"}'


def test_write_acl():
    acl = ACL.get_write_acl("ROLE_TEST", as_list=False)
    assert str(acl) == '{"allow": true, "role": "ROLE_TEST", "action": "write"}'


def test_read_write_acl():
    acls = ACL.get_read_write_acls("ROLE_TEST")
    assert len(acls) == 2
    assert str(acls[0]) == '{"allow": true, "role": "ROLE_TEST", "action": "read"}'
    assert str(acls[1]) == '{"allow": true, "role": "ROLE_TEST", "action": "write"}'



#  Copyright (c) 2019. Tobias Kurze

import json
import re
from loguru import logger
from pprint import pprint
from typing import List, Optional

d = {
    "@id": "68e9fcb3-0662-43dd-a957-ba9bc1414195",
    "@type": "presentation/source",
    "audio": {
        "@id": "audio-1",
        "bitrate": "83050.0",
        "channels": "1",
        "device": None,
        "encoder": {
            "@type": "AAC (Advanced Audio Coding)"
        },
        "framecount": "261",
        "samplingrate": "48000"
    },
    "checksum": {
        "#text": "a3ac7ddabb263c2d00b73e8177d15c8d",
        "@type": "md5"
    },

    "duration": "5568",
    "live": "false",
    "mimetype": "video/mp4",
    "size": "383631",
    "tags": {
        "tag": "archive"
    },
    "url": "https://oc-bib-admin.bibliothek.kit.edu/assets/assets/983d0f7c-f81c-473a-80c2-e5a9b497ae99/68e9fcb3-0662-43dd-a957-ba9bc1414195/3/vid1_hqt.mp4",
    "video": {
        "@id": "video-1",
        "bitrate": "465641.0",
        "device": None,
        "encoder": {
            "@type": "H.264 / AVC / MPEG-4 AVC / MPEG-4 part 10"
        },
        "framecount": "166",
        "framerate": "30.0",
        "resolution": "560x320"
    }
}


class Track:
    def __init__(self, data):
        self.data = data
        self._fix_dict_keys()

    def __str__(self):
        return json.dumps(self.data)

    def __repr__(self):
        return self.__str__()

    def _fix_dict_keys(self, d: Optional[dict] = None):
        if d is None:
            d = self.data
        try: 
            problematic_keys: List[str] = [k for k in d.keys() if any([s in k for s in ['#', '@']])]
            for p_k in problematic_keys:
                nice_key = p_k.replace('#', '').replace('@', '')
                d[nice_key] = d.pop(p_k)
            for k in d.keys():  # recurse
                if isinstance(d[k], dict):
                    self._fix_dict_keys(d[k])
        except AttributeError as e:
            logger.error(f"AttributeError: {e}")

    def get_type(self):
        return self.data.get("type", None)

    def get_audio(self):
        return self.data.get("audio", None)

    def get_video(self):
        return self.data.get("video", None)


class TrackList:
    def __init__(self, data: List[dict]):
        self.tracks = []
        for t in data:
            if isinstance(t, dict):
                self.tracks.append(Track(t))

    def get_tracks(self) -> List[Track]:
        return self.tracks

    def get_tracks_by_flavor(self, flavor: re = r'.*/source') -> List[Track]:
        tracks = []
        for t in self.tracks:
            t_type = t.get_type()
            regexp = re.compile(flavor)
            if regexp.search(t_type):
                tracks.append(t)
        return tracks


if __name__ == '__main__':
    pprint(d)
    pprint(Track(d).data)



"POST: https://oc9-dev-admin.bibliothek.kit.edu/admin-ng/event/new"
import json
from datetime import datetime
from types import NoneType

from loguru import logger
from typing import Union, Optional, List, Dict

import pendulum
import requests
from pendulum import DateTime
from requests.auth import HTTPBasicAuth

from ocpy import OcPyRequestException
from ocpy.api.api_client import OpenCastBaseApiClient
from ocpy.model.mediapackage import MediaPackage

"""
{
"metadata":[
    {
        "flavor":"dublincore/episode","title":"EVENTS.EVENTS.DETAILS.CATALOG.EPISODE",
        "fields":[{"id":"title","type":"text","value":"lala","tabindex":1,"$$hashKey":"object:544"},{"id":"subject","type":"text","value":"","tabindex":2,"$$hashKey":"object:545"},{"id":"description","type":"text_long","value":"","tabindex":3,"$$hashKey":"object:546"},{"translatable":true,"id":"language","type":"text","value":"","tabindex":4,"$$hashKey":"object:547"},{"id":"rightsHolder","type":"text","value":"","tabindex":5,"$$hashKey":"object:548"},{"translatable":true,"id":"license","type":"ordered_text","value":"","tabindex":6,"$$hashKey":"object:549"},{"translatable":false,"id":"isPartOf","type":"text","value":"","tabindex":7,"$$hashKey":"object:550"},{"translatable":false,"id":"creator","type":"mixed_text","value":[],"tabindex":8,"$$hashKey":"object:551"},{"translatable":false,"id":"contributor","type":"mixed_text","value":[],"tabindex":9,"$$hashKey":"object:552"},{"id":"publisher","type":"text","value":"Administrator","tabindex":10,"$$hashKey":"object:553"}]
    },
    {
        "fields":[{"translatable":true,"id":"content_type","type":"text","value":"","tabindex":1,"$$hashKey":"object:703"},{"id":"doi","type":"text","value":"","tabindex":2,"$$hashKey":"object:704"},{"id":"date","type":"date","value":"","tabindex":3,"$$hashKey":"object:705"},{"id":"institution","type":"text","value":"","tabindex":4,"$$hashKey":"object:706"},{"id":"keywords","type":"mixed_text","value":[],"tabindex":5,"$$hashKey":"object:707"},{"translatable":false,"id":"subject_classification","type":"text","value":"","tabindex":6,"$$hashKey":"object:708"}],
        "flavor":"kit/episode","title":"EVENTS.EVENTS.DETAILS.CATALOG.EPISODE"
    }
],
"processing":{"workflow":"fast","configuration":{"publish":"true"}},
"access":{"acl":{"ace":[{"action":"read","allow":true,"role":"ROLE_USER_ADMIN"},{"action":"write","allow":true,"role":"ROLE_USER_ADMIN"}]}},
"source":{
    "type":"SCHEDULE_MULTIPLE",
    "metadata":
    {
        "start":"2022-06-21T11:00:00Z",
        "device":"Test-SMP_KIT",
        "inputs":"defaults",
        "duration":"3300000",
        "end":"2022-06-21T11:55:00Z",
        "rrule":"FREQ=WEEKLY;BYDAY=TU,FR;BYHOUR=11;BYMINUTE=0"
    }
},
"assets":{"workflow":"publish-uploaded-assets","options":[{"id":"track_presenter","type":"track","flavorType":"presenter","flavorSubType":"source","multiple":false,"displayOrder":13,"accept":".avi,.flv,.m4v,.mkv,.mov,.mp4,.mpeg,.mpg,.ogv,.webm,.wmv,.flac,.m4a,.mp3,.ogg,.wav,.wma","title":"EVENTS.EVENTS.NEW.SOURCE.UPLOAD.NON_SEGMENTABLE"},{"id":"track_audio","type":"track","flavorType":"presenter-audio","flavorSubType":"source","multiple":false,"displayOrder":12,"accept":".flac,.m4a,.mp3,.ogg,.wav,.wma","title":"EVENTS.EVENTS.NEW.SOURCE.UPLOAD.AUDIO_ONLY"},{"id":"track_presentation","type":"track","flavorType":"presentation","flavorSubType":"source","multiple":false,"displayOrder":14,"accept":".avi,.flv,.m4v,.mkv,.mov,.mp4,.mpeg,.mpg,.ogv,.webm,.wmv,.flac,.m4a,.mp3,.ogg,.wav,.wma","title":"EVENTS.EVENTS.NEW.SOURCE.UPLOAD.SEGMENTABLE"},{"id":"attachment_captions_webvtt","type":"attachment","flavorType":"text","flavorSubType":"vtt","displayOrder":3,"accept":".vtt","title":"EVENTS.EVENTS.NEW.UPLOAD_ASSET.OPTION.CAPTIONS_WEBVTT"}]}}
"""


class RecordingsApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/recordings"

    def get_event_count(self):
        url = self.base_url + "/eventCount"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if res.ok:
            return res.text
        raise OcPyRequestException(res.content.decode("utf-8"),
                                   response=res)

    def get_upcoming_recording(self, agent_id: str):
        url = self.base_url + f"/capture/{agent_id}/upcoming"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if res.ok:
            return res.json()
        raise OcPyRequestException(res.content.decode("utf-8"), response=res)

    @staticmethod
    def _build_default_agent_parameters_dict(agent_parameters: Optional[Dict[str, str]] = None,
                                             workflow_id: str = "fast",
                                             capture_device_names: str = "defaults"):
        if agent_parameters is None:
            agent_parameters = {}

        if "capture.device.names" not in agent_parameters and capture_device_names is not None:
            agent_parameters["capture.device.names"] = capture_device_names

        if "org.opencastproject.workflow.definition" not in agent_parameters and workflow_id is not None:
            agent_parameters["org.opencastproject.workflow.definition"] = workflow_id

        return agent_parameters

    @staticmethod
    def _build_properties_string(properties_dict_or_list: Union[Dict, List]) -> str:
        logger.debug(properties_dict_or_list)
        if isinstance(properties_dict_or_list, dict):
            properties_dict_or_list = [f"{k}={properties_dict_or_list[k]}" for k in properties_dict_or_list.keys()]
        if isinstance(properties_dict_or_list, list):
            properties_dict_or_list = "\n".join(properties_dict_or_list)
        if not isinstance(properties_dict_or_list, (str, NoneType)):
            logger.warning(f"properties_dict_or_list is not a string; neither string, nor dict nor list was provided! "
                           f"({type(properties_dict_or_list)})")

        return properties_dict_or_list

    def create_recording(self, agent_id: str, start: Union[str, DateTime], end: Union[str, DateTime],
                         media_package: str, source: Optional[str] = None, users: Optional[str] = None,
                         wf_properties: Optional[Dict[str, str]] = None,
                         agent_parameters: Optional[Dict[str, str]] = None,
                         workflow_id: str = "fast", capture_device_names: str = "defaults"):

        if isinstance(start, str):
            start = pendulum.parse(start)
        if isinstance(end, str):
            end = pendulum.parse(end)

        agent_parameters = RecordingsApi._build_default_agent_parameters_dict(agent_parameters,
                                                                              workflow_id=workflow_id,
                                                                              capture_device_names=capture_device_names)

        data = {
            "start": int(start.timestamp() * 1000),
            "end": int(end.timestamp() * 1000),
            "agent": agent_id,
            "mediaPackage": media_package,
            "users": users,
            "wfproperties": RecordingsApi._build_properties_string(wf_properties),
            "agentparameters": RecordingsApi._build_properties_string(agent_parameters),
            "source": source,
        }

        logger.info(data)

        res = requests.post(
            self.base_url + "/",
            auth=HTTPBasicAuth(self.user, self.password),
            data=data,
        )
        if res.ok:
            print(res)
            rec = res.text
            return rec
        raise OcPyRequestException(res.content.decode("utf-8"), response=res)

    def create_recurring_recording(self, agent_id: str, start: Union[str, DateTime], end: Union[str, DateTime],
                                   duration: int, rrule: str, template_mp: str, tz: str = "Europe/Berlin",
                                   source: Optional[str] = None,
                                   users: Optional[List[str]] = None,
                                   wf_properties: Optional[Dict[str, str]] = None,
                                   agent_parameters: Optional[Dict[str, str]] = None, workflow_id: str = "fast",
                                   capture_device_names: str = "defaults",
                                   **kwargs):

        if isinstance(start, str):
            start = pendulum.parse(start)
        if isinstance(end, str):
            end = pendulum.parse(end)

        agent_parameters = RecordingsApi._build_default_agent_parameters_dict(agent_parameters,
                                                                              workflow_id=workflow_id,
                                                                              capture_device_names=capture_device_names)
        data = {
            "rrule": rrule,
            "start": int(start.timestamp() * 1000),
            "end": int(end.timestamp() * 1000),
            "duration": duration,
            "tz": tz,
            "agent": agent_id,
            "templateMp": template_mp,
            "users": users,
            "wfproperties": RecordingsApi._build_properties_string(wf_properties),
            "agentparameters": RecordingsApi._build_properties_string(agent_parameters),
            "source": source,
        }

        res = requests.post(
            self.base_url + "/multiple",
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            data=data,
        )
        if res.ok:
            return "ok"
        raise OcPyRequestException(res.content.decode("utf-8"), response=res)


def main():
    api = RecordingsApi()
    print(api.get_event_count())


if __name__ == "__main__":
    main()



#  Copyright (c) 2019. Tobias Kurze

import requests
from urllib.parse import urlparse
from requests.auth import HTTPBasicAuth
import json
from datetime import datetime

from typing import List, Optional

from ocpy import OcPyRequestException
from ocpy.api.api_client import OpenCastBaseApiClient
from ocpy.model.organization import Organization


class User:
    def __init__(self, user, password, url, data):
        self.base_url = url
        self.user = user
        self.password = password
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_name(self) -> str:
        return self.data["name"]

    def get_username(self) -> str:
        return self.data["username"]

    def get_email(self) -> Optional[str]:
        try:
            return self.data["email"]
        except KeyError:
            return None

    def get_provider(self):
        return self.data["provider"]

    def get_manageable(self) -> bool:
        return self.data["manageable"]

    def get_roles(self):
        return self.data["roles"]

    def get_organization(self):
        data = self.data["organization"]
        data["admin_role"] = data["adminRole"]
        del data["adminRole"]
        data["anonymous_role"] = data["anonymousRole"]
        del data["anonymousRole"]
        return Organization(**self.data["organization"])


class UsersApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, internal_api=True, **kwargs):
        """
        :param user:
        :param password:
        :param server_url:
        :param internal_api: if set to false, also external users will be displayed
        """
        super().__init__(user, password, server_url)
        if internal_api:
            self.base_url = self.server_url + "/user-utils"
        else:
            self.base_url = self.server_url + "/users"

    def get_users(
        self, method="GET", limit=100, offset=0, query="", **kwargs
    ) -> List[User]:
        """
        :param method:
        :param limit:
        :param offset:
        :param query:
        :param kwargs:
        :return: List[Series]
        """
        parameters = {"limit": limit, "offset": offset, "query": query}

        results = []
        print(self.base_url + "/users.json")
        res = requests.request(
            method,
            self.base_url + "/users.json",
            auth=HTTPBasicAuth(self.user, self.password),
            params=parameters,
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            for u in res.json()["users"]["user"]:
                results.append(
                    User(
                        self.user,
                        self.password,
                        self.base_url + "/" + u["username"] + ".json",
                        u,
                    )
                )
        else:
            raise OcPyRequestException(res.content.decode("utf-8"), res.status_code)
        return results

    def get_user_by_name(self, user_name, **kwargs) -> User:
        res = requests.get(
            self.base_url + "/" + user_name + ".json",
            auth=HTTPBasicAuth(self.user, self.password),
            **kwargs
        )
        if res.ok:
            user = res.json()
            return User(
                self.user, self.password, self.base_url + "/" + user["identifier"], user
            )
        raise Exception("could not get event!")

    def create_user(
        self,
        user_name: str,
        password: str,
        name: Optional[str] = None,
        email: Optional[str] = None,
        roles: Optional[List[str]] = None,
        **kwargs
    ):
        if "user-utils" not in self.base_url:
            raise NotImplementedError("can't create user when using external API.")

        res = requests.delete(
            self.base_url + "/" + user_name + ".json",
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )

    def delete_user(self, user_name: str, **kwargs):
        if "user-utils" not in self.base_url:
            raise NotImplementedError("can't delete user when using external API.")

        res = requests.delete(
            self.base_url + "/" + user_name + ".json",
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
        )
        if res.ok:
            return "OK"

    def update_user(self, user_name: str):
        if "user-utils" not in self.base_url:
            raise NotImplementedError("can't update user when using external API.")


def main():
    api = UsersApi(internal_api=True)
    # api.create_user()
    users = api.get_users()
    # print(users)

    for u in users:
        print(u.get_name())
        print(u.get_username())
        # print(u.get_roles())
        print(u.get_manageable())


if __name__ == "__main__":
    main()
